@@ -450,8 +450,8 @@ gst_a52dec_chain (GstPad * pad, GstData * _data)
     a52dec->time = GST_BUFFER_TIMESTAMP (buf);
     GST_DEBUG_OBJECT (a52dec,
         "Received buffer with ts %" GST_TIME_FORMAT " duration %"
-        GST_TIME_FORMAT " size %d", GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)), GST_BUFFER_SIZE (buf));
+        GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
   }
 
   if (a52dec->cache) {
