@@ -143,8 +143,7 @@ enum
   ARG_BUFFERSIZE,
   ARG_AUTORECOVER,
   ARG_MMAP,
-  ARG_MAXDISCONT,
-  ARG_ADVANCED_PROBING
+  ARG_MAXDISCONT
 };
 
 static GstElement *parent_class = NULL;
@@ -177,7 +176,7 @@ gst_alsa_class_init (gpointer g_class, gpointer class_data)
           "Name of the device", NULL, G_PARAM_READABLE));
   g_object_class_install_property (object_class, ARG_PERIODCOUNT,
       g_param_spec_int ("period-count", "Period count",
-          "Number of hardware buffers to use (0 for card's default)",
+          "Number of hardware buffers to use",
           GST_ALSA_MIN_PERIOD_CNT, GST_ALSA_MAX_PERIOD_CNT,
           GST_ALSA_MIN_PERIOD_CNT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, ARG_PERIODSIZE,
@@ -196,18 +195,13 @@ gst_alsa_class_init (gpointer g_class, gpointer class_data)
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, ARG_MMAP,
       g_param_spec_boolean ("mmap", "Use mmap'ed access",
-          "Whether to use mmap (faster) or standard read/write (more compatible)",
+          "Wether to use mmap (faster) or standard read/write (more compatible)",
           TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, ARG_MAXDISCONT,
       g_param_spec_uint64 ("max-discont", "Maximum Discontinuity",
           "GStreamer timeunits before the timestamp syncing starts dropping/inserting samples",
           /* rounding errors */ 1000, GST_SECOND, GST_ALSA_DEFAULT_DISCONT,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-  g_object_class_install_property (object_class, ARG_ADVANCED_PROBING,
-      g_param_spec_boolean ("advanced-probing", "Advanced Probing",
-          "When using the default device, try to probe the underlying device "
-          "(this should give more narrowly defined caps in many cases)",
-          TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
   element_class->change_state = GST_DEBUG_FUNCPTR (gst_alsa_change_state);
   element_class->query = GST_DEBUG_FUNCPTR (gst_alsa_query);
@@ -221,7 +215,6 @@ gst_alsa_init (GstAlsa * this)
 {
   this->device = g_strdup ("default");
   this->cached_caps = NULL;
-  this->period_count = 2;
 
   GST_FLAG_SET (this, GST_ELEMENT_EVENT_AWARE);
   GST_FLAG_SET (this, GST_ELEMENT_THREAD_SUGGESTED);
@@ -265,16 +258,16 @@ gst_alsa_set_property (GObject * object, guint prop_id, const GValue * value,
       break;
     case ARG_PERIODCOUNT:
       g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
-      this->desired_period_count = g_value_get_int (value);
+      this->period_count = g_value_get_int (value);
       break;
     case ARG_PERIODSIZE:
       g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
-      this->desired_period_size = g_value_get_int (value);
+      this->period_size = g_value_get_int (value);
       break;
     case ARG_BUFFERSIZE:
       g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
       buffer_size = g_value_get_int (value);
-      this->desired_period_count = buffer_size / this->desired_period_size;
+      this->period_count = buffer_size / this->period_size;
       break;
     case ARG_AUTORECOVER:
       this->autorecover = g_value_get_boolean (value);
@@ -285,10 +278,6 @@ gst_alsa_set_property (GObject * object, guint prop_id, const GValue * value,
     case ARG_MAXDISCONT:
       this->max_discont = (GstClockTime) g_value_get_uint64 (value);
       return;
-    case ARG_ADVANCED_PROBING:
-      g_return_if_fail (!GST_FLAG_IS_SET (this, GST_ALSA_RUNNING));
-      this->advanced_probing = g_value_get_boolean (value);
-      return;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       return;
@@ -322,11 +311,10 @@ gst_alsa_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_int (value, this->period_count);
       break;
     case ARG_PERIODSIZE:
-      g_value_set_int (value, this->desired_period_size);
+      g_value_set_int (value, this->period_size);
       break;
     case ARG_BUFFERSIZE:
-      g_value_set_int (value,
-          this->desired_period_size * this->desired_period_count);
+      g_value_set_int (value, this->period_size * this->period_count);
       break;
     case ARG_AUTORECOVER:
       g_value_set_boolean (value, this->autorecover);
@@ -337,9 +325,6 @@ gst_alsa_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_MAXDISCONT:
       g_value_set_uint64 (value, (guint64) this->max_discont);
       return;
-    case ARG_ADVANCED_PROBING:
-      g_value_set_boolean (value, this->advanced_probing);
-      return;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -598,7 +583,6 @@ gst_alsa_get_format (const GstStructure * structure)
 {
   const gchar *mimetype;
   GstAlsaFormat *ret;
-  gint rate, chans;
 
   if (!(ret = g_new (GstAlsaFormat, 1)))
     return NULL;
@@ -662,13 +646,10 @@ gst_alsa_get_format (const GstStructure * structure)
   }
 
   /* get rate and channels */
-  if (!gst_structure_get_int (structure, "rate", &rate)
-      || !gst_structure_get_int (structure, "channels", &chans))
+  if (!(gst_structure_get_int (structure, "rate", &ret->rate) &&
+          gst_structure_get_int (structure, "channels", &ret->channels)))
     goto error;
 
-  ret->rate = rate;
-  ret->channels = chans;
-
   return ret;
 
 error:
@@ -832,39 +813,6 @@ gst_alsa_caps (snd_pcm_format_t format, gint rate, gint channels)
   return ret_caps;
 }
 
-gboolean
-gst_alsa_set_default_format (GstAlsa * this)
-{
-  GstAlsaFormat *ret = NULL;
-
-  if (this->handle == NULL)
-    if (!gst_alsa_open_audio (this))
-      return FALSE;
-
-  /* Attempt to set a default format, fail if we can't */
-  ret = g_new (GstAlsaFormat, 1);
-  if (ret != NULL) {
-    ret->format = snd_pcm_build_linear_format (16, 16, 0, 0);
-    ret->rate = 44100;
-    ret->channels = 2;
-    if (gst_alsa_probe_hw_params (this, ret)) {
-      this->format = ret;
-
-      if (GST_FLAG_IS_SET (this, GST_ALSA_RUNNING))
-        gst_alsa_stop_audio (this);
-
-      if (gst_alsa_start_audio (this)) {
-        return TRUE;
-      }
-
-      this->format = NULL;
-    }
-    /* Give up too easily */
-    g_free (ret);
-  }
-
-  return FALSE;
-}
 
 static int
 gst_alsa_check_sample_rates (snd_pcm_t * device_handle,
@@ -1001,14 +949,10 @@ gst_alsa_get_caps (GstPad * pad)
 
   this = GST_ALSA (gst_pad_get_parent (pad));
 
-  if (!GST_FLAG_IS_SET (this, GST_ALSA_OPEN)) {
-    GST_DEBUG ("device not open, returning template caps");
+  if (!GST_FLAG_IS_SET (this, GST_ALSA_OPEN))
     return gst_caps_copy (GST_PAD_TEMPLATE_CAPS (GST_PAD_PAD_TEMPLATE (pad)));
-  }
-  if (this->cached_caps) {
-    GST_DEBUG ("returning cached caps");
+  if (this->cached_caps)
     return gst_caps_copy (this->cached_caps);
-  }
 
   snd_pcm_hw_params_alloca (&hw_params);
   ERROR_CHECK (snd_pcm_hw_params_any (this->handle, hw_params),
@@ -1018,14 +962,14 @@ gst_alsa_get_caps (GstPad * pad)
     min_channels = 1;
     max_channels = -1;
   } else {
-    guint min, max;
-
-    ERROR_CHECK (snd_pcm_hw_params_get_channels_min (hw_params, &min),
+    ERROR_CHECK (snd_pcm_hw_params_get_channels_min (hw_params, &min_channels),
         "Couldn't get minimum channel count for device %s: %s", this->device);
-    ERROR_CHECK (snd_pcm_hw_params_get_channels_max (hw_params, &max),
+    ERROR_CHECK (snd_pcm_hw_params_get_channels_max (hw_params, &max_channels),
         "Couldn't get maximum channel count for device %s: %s", this->device);
-    min_channels = (min < 1) ? 1 : min;
-    max_channels = (max > GST_ALSA_MAX_CHANNELS) ? GST_ALSA_MAX_CHANNELS : max;
+    min_channels = min_channels < 1 ? 1 : min_channels;
+    max_channels =
+        max_channels >
+        GST_ALSA_MAX_CHANNELS ? GST_ALSA_MAX_CHANNELS : max_channels;
   }
 
   /* Check available sample rates */
@@ -1100,15 +1044,13 @@ gst_alsa_get_caps (GstPad * pad)
     this->period_size = GST_ALSA_MAX_BUFFER_SZ / this->period_count;
   };
 
+
   snd_pcm_format_mask_alloca (&mask);
   snd_pcm_hw_params_get_format_mask (hw_params, mask);
-
   for (i = 0; i <= SND_PCM_FORMAT_LAST; i++) {
     if (snd_pcm_format_mask_test (mask, i)) {
       GstCaps *caps = gst_alsa_get_caps_internal (i);
 
-      GST_DEBUG ("supported format: %02d %s", i, snd_pcm_format_name (i));
-
       /* we can never use a format we can't set caps for */
       if (caps != NULL) {
         gint n;
@@ -1176,7 +1118,7 @@ gst_alsa_get_caps (GstPad * pad)
     ret = gst_caps_new_empty ();
   }
   gst_caps_do_simplify (ret);
-  GST_LOG_OBJECT (this, "get_caps returns %" GST_PTR_FORMAT, ret);
+  GST_LOG_OBJECT (this, "get_caps returns %P", ret);
 
   g_value_unset (&supported_rates);
 
@@ -1622,84 +1564,11 @@ gst_alsa_set_eos (GstAlsa * this)
   gst_element_set_eos (GST_ELEMENT (this));
 }
 
-static gboolean
-gst_alsa_open_audio_device (GstAlsa * this, const gchar * device,
-    gboolean ignore_error)
-{
-  snd_pcm_stream_t stream_type;
-  snd_pcm_t *handle;
-  int ret;
-
-  stream_type = GST_ALSA_GET_CLASS (this)->stream;
-  ret = snd_pcm_open (&handle, device, stream_type, SND_PCM_NONBLOCK);
-  if (ret == 0) {
-    GST_DEBUG_OBJECT (this, "opened device '%s'", device);
-    this->handle = handle;
-    return TRUE;
-  }
-
-  if (ignore_error)
-    return FALSE;
-
-  /* ALSA inverts standard errno.h error codes */
-  switch (-ret) {
-    case EBUSY:
-      GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
-          (_("ALSA device \"%s\" is already in use by another program."),
-              this->device), (NULL));
-      break;
-    case EACCES:
-    case ETXTBSY:
-      GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, OPEN_READ_WRITE,
-          (_("Could not access ALSA device \"%s\", check its permissions."),
-              this->device), GST_ERROR_SYSTEM);
-      break;
-
-    case ENXIO:
-    case ENODEV:
-    case ENOENT:
-      GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, NOT_FOUND,
-          (_("ALSA device \"%s\" does not exist."), this->device), (NULL));
-      break;
-    default:
-      GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, OPEN_READ_WRITE,
-          (_("ALSA device \"%s\" had an error."),
-              this->device), ("ALSA error %d: %s", ret, snd_strerror (ret)));
-      break;
-  }
-  return FALSE;
-}
-
-static gchar *
-gst_snd_pcm_info_get_real_device (snd_pcm_info_t * info)
-{
-  const gchar *id, *name, *subdev_name, *s;
-  gint card_num;
-
-  id = snd_pcm_info_get_id (info);
-  name = snd_pcm_info_get_name (info);
-  subdev_name = snd_pcm_info_get_subdevice_name (info);
-
-  GST_DEBUG ("PCM info: id='%s', name='%s', subdevice_name='%s', card=%d, "
-      "subdevice=%u", GST_STR_NULL (id), GST_STR_NULL (name),
-      GST_STR_NULL (subdev_name), snd_pcm_info_get_card (info),
-      snd_pcm_info_get_subdevice (info));
-
-  if (!id || *id == '\0')
-    return NULL;
-
-  s = strchr (id, ':');
-  if (!s || sscanf (s, ":%d", &card_num) != 1)
-    return NULL;
-
-  return g_strdup (id);
-}
-
 static gboolean
 gst_alsa_open_audio (GstAlsa * this)
 {
   snd_pcm_info_t *info;
-  gchar *real_dev;
+  int ret;
 
   g_assert (this != NULL);
   g_assert (this->handle == NULL);
@@ -1716,42 +1585,42 @@ gst_alsa_open_audio (GstAlsa * this)
       "error opening log output: %s");
 #endif
 
-  if (!gst_alsa_open_audio_device (this, this->device, FALSE))
+  if ((ret = snd_pcm_open (&this->handle, this->device,
+              GST_ALSA_GET_CLASS (this)->stream, SND_PCM_NONBLOCK)) < 0) {
+    /* ALSA inverts standard errno.h error codes */
+    switch (-ret) {
+      case EBUSY:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+            (_("ALSA device \"%s\" is already in use by another program."),
+                this->device), (NULL));
+        break;
+      case EACCES:
+      case ETXTBSY:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, OPEN_READ_WRITE,
+            (_("Could not access ALSA device \"%s\", check its permissions."),
+                this->device), GST_ERROR_SYSTEM);
+        break;
+
+      case ENXIO:
+      case ENODEV:
+      case ENOENT:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+            (_("ALSA device \"%s\" does not exist."), this->device), (NULL));
+        break;
+      default:
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+            (_("ALSA device \"%s\" had an error."),
+                this->device), ("ALSA error %d: %s", ret, snd_strerror (ret)));
+        break;
+    }
     return FALSE;
+  }
 
   snd_pcm_info_malloc (&info);
   snd_pcm_info (this->handle, info);
   this->cardname = g_strdup (snd_pcm_info_get_name (info));
-  if (this->advanced_probing && strcmp (this->device, "default") == 0) {
-    real_dev = gst_snd_pcm_info_get_real_device (info);
-  } else {
-    real_dev = NULL;
-  }
   snd_pcm_info_free (info);
 
-  /* If we have opened the default device, chances are that there's a plughw
-   * layer somewhere between us and the driver and that layer will claim it
-   * can do almost any format in the world, while we really just want the
-   * formats we can use directly; so what we do here is to try and open the
-   * underlying device of the 'default' device instead, which should give
-   * us much better results when probing the device capabilities later on */
-
-  if (real_dev != NULL) {
-    GST_DEBUG ("Closing %s, opening %s instead", this->device, real_dev);
-    snd_pcm_close (this->handle);
-    this->handle = NULL;
-    if (!gst_alsa_open_audio_device (this, real_dev, TRUE)) {
-      GST_DEBUG ("Cannot open %s, reverting to %s", real_dev, this->device);
-      if (!gst_alsa_open_audio_device (this, this->device, FALSE)) {
-        g_free (real_dev);
-        return FALSE;
-      }
-    } else {
-      GST_DEBUG ("Opened device %s", real_dev);
-    }
-    g_free (real_dev);
-  }
-
   GST_FLAG_SET (this, GST_ALSA_OPEN);
   return TRUE;
 }
@@ -1779,6 +1648,7 @@ static gboolean
 gst_alsa_probe_hw_params (GstAlsa * this, GstAlsaFormat * format)
 {
   snd_pcm_hw_params_t *hw_params;
+  snd_pcm_uframes_t period_size;
   unsigned int period_count;
   unsigned int rate;
 
@@ -1823,16 +1693,12 @@ gst_alsa_probe_hw_params (GstAlsa * this, GstAlsaFormat * format)
     GST_WARNING_OBJECT (this, "set rate (%u) differs from desired rate (%u)",
         rate, format->rate);
 
-  if (this->desired_period_count > 0) {
-    period_count = this->desired_period_count;
-    SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle,
-            hw_params, &period_count, 0));
-  }
-  this->period_size = this->desired_period_size;
+  period_count = this->period_count;
+  SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle,
+          hw_params, &period_count, 0));
+  period_size = this->period_size;
   SIMPLE_ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle,
-          hw_params, &this->period_size, 0));
-  GST_DEBUG ("Desired period size: %d, actual period size: %d",
-      this->desired_period_size, this->period_size);
+          hw_params, &period_size, 0));
 
   return TRUE;
 }
@@ -1847,7 +1713,6 @@ gst_alsa_set_hw_params (GstAlsa * this)
 {
   snd_pcm_hw_params_t *hw_params;
   unsigned int rate;
-  int ret;
 
   g_return_val_if_fail (this != NULL, FALSE);
   g_return_val_if_fail (this->handle != NULL, FALSE);
@@ -1892,31 +1757,13 @@ gst_alsa_set_hw_params (GstAlsa * this)
     if (rate != this->format->rate)
       GST_WARNING_OBJECT (this, "set rate (%u) differs from desired rate (%u)",
           rate, this->format->rate);
-    this->period_count =
-        this->desired_period_count ? this->desired_period_count : 2;
-    ret =
-        snd_pcm_hw_params_set_periods_near (this->handle, hw_params,
-        &this->period_count, 0);
-    if (ret < 0) {
-      /* if there's a specific period count requested, we have an error now. 
-       * Otherwise, we just don't care */
-      if (this->desired_period_count) {
-        GST_ERROR ("could not set period count to %u: %s",
-            (guint) this->period_count, snd_strerror (ret));
-        return FALSE;
-      } else {
-        GST_INFO ("could not set period count to %u: %s - using default",
-            (guint) this->period_count, snd_strerror (ret));
-      }
-    }
-
-    this->period_size = this->desired_period_size;
+    ERROR_CHECK (snd_pcm_hw_params_set_periods_near (this->handle, hw_params,
+            &this->period_count, 0), "error setting period count to %u: %s",
+        (guint) this->period_count);
     ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle,
             hw_params, &this->period_size, 0),
         "error setting period size to %u frames: %s",
         (guint) this->period_size);
-    GST_DEBUG ("Desired period size: %d, actual period size: %d",
-        this->desired_period_size, this->period_size);
   } else {
     GST_INFO_OBJECT (this, "Preparing format: (none)");
   }
@@ -1926,8 +1773,6 @@ gst_alsa_set_hw_params (GstAlsa * this)
   ERROR_CHECK (snd_pcm_hw_params (this->handle, hw_params),
       "Could not set hardware parameters: %s");
 
-  ERROR_CHECK (snd_pcm_hw_params_get_periods (hw_params, &this->period_count,
-          0), "Could not get period count: %s");
   /* now get the pcm caps */
   GST_ALSA_CAPS_SET (this, GST_ALSA_CAPS_PAUSE,
       snd_pcm_hw_params_can_pause (hw_params));
@@ -2086,8 +1931,6 @@ gst_alsa_close_audio (GstAlsa * this)
     gst_caps_free (this->cached_caps);
     this->cached_caps = NULL;
   }
-  this->period_count =
-      this->desired_period_count ? this->desired_period_count : 2;
 
   return TRUE;
 }
