@@ -218,7 +218,7 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
         break;
       case GST_EVENT_DISCONTINUOUS:
       {
-        GstClockTime value, delay;
+        GstClockTime value = GST_CLOCK_TIME_NONE, delay;
 
         /* only the first pad may seek */
         if (pad_nr != 0) {
@@ -246,8 +246,45 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
               "couldn't extract time from discont event. Bad things might happen!");
         }
 
+        if (GST_CLOCK_TIME_IS_VALID (value)) {
+          GST_DEBUG ("Set discont to %" GST_TIME_FORMAT, GST_TIME_ARGS (value));
+        }
+
         break;
       }
+      case GST_EVENT_FILLER:{
+        guint64 dur = gst_event_filler_get_duration (event);
+
+        if (GST_CLOCK_TIME_IS_VALID (dur)) {
+          gint samples;
+          GstBuffer *buf;
+
+          buf = gst_buffer_new ();
+          GST_BUFFER_TIMESTAMP (buf) = GST_EVENT_TIMESTAMP (event);
+          GST_BUFFER_DURATION (buf) = dur;
+
+          samples = gst_alsa_timestamp_to_samples (this,
+              GST_BUFFER_DURATION (buf));
+          if (GST_ELEMENT (this)->numsinkpads == 1)
+            samples *= this->format->channels;
+          GST_BUFFER_SIZE (buf) = samples *
+              snd_pcm_format_physical_width (this->format->format) / 8;
+          GST_BUFFER_DATA (buf) = g_malloc (GST_BUFFER_SIZE (buf));
+          if (0 != snd_pcm_format_set_silence (this->format->format,
+                  GST_BUFFER_DATA (buf), samples)) {
+            GST_WARNING_OBJECT (this, "error silencing buffer, may hang");
+            gst_buffer_unref (buf);
+          } else {
+            GST_DEBUG_OBJECT (this, "adding %" GST_TIME_FORMAT " silence at %"
+                GST_TIME_FORMAT, GST_TIME_ARGS (dur),
+                GST_TIME_ARGS (GST_EVENT_TIMESTAMP (event)));
+            sink->gst_data[pad_nr] = GST_DATA (buf);
+          }
+        }
+        break;
+      }
+      case GST_EVENT_TAG:
+        break;
       default:
         GST_INFO_OBJECT (this, "got an unknown event (Type: %d)",
             GST_EVENT_TYPE (event));
@@ -256,7 +293,6 @@ gst_alsa_sink_check_event (GstAlsaSink * sink, gint pad_nr)
     GST_LOG_OBJECT (sink, "unreffing event %p of type %d with refcount %d",
         event, GST_EVENT_TYPE (event), GST_DATA_REFCOUNT (event));
     gst_event_unref (event);
-    sink->gst_data[pad_nr] = NULL;
   } else {
     /* the element at the top of the chain did not emit an event. */
     g_assert_not_reached ();
@@ -405,7 +441,7 @@ sink_restart:
          * assumes that both calls return the same value. However they can be
          * wildly different, since snd_pcm_delay goes deep into the kernel.
          */
-        if (gst_element_get_clock (element) == GST_CLOCK (this->clock)) {
+        if (this->ext_clock == GST_CLOCK (this->clock)) {
           /* FIXME: this is ugly because of the variables it uses but I
            * don't know a better way to get this info */
           if (element->base_time > this->clock->start_time) {
@@ -440,9 +476,15 @@ sink_restart:
           /* there are empty samples in front of us, fill them with silence */
           int samples = MIN (bytes, sample_diff) *
               (element->numpads == 1 ? this->format->channels : 1);
-          int size =
-              samples * snd_pcm_format_physical_width (this->format->format) /
-              8;
+          int width = snd_pcm_format_physical_width (this->format->format);
+          int size = samples * width / 8;
+
+          if (size / (width / 8) != samples || samples > max_discont) {
+            GST_WARNING_OBJECT (this,
+                "Integer overflow for size=%d/samples=%d (sample_diff=%ld) - broken stream",
+                size, samples, sample_diff);
+            goto no_difference;
+          }
           GST_INFO_OBJECT (this,
               "Allocating %d bytes (%ld samples) now to resync: sample %lu expected, but got %ld",
               size, MIN (bytes, sample_diff), expected, samplestamp);
