@@ -60,8 +60,8 @@ static GstStaticPadTemplate sink_temp = GST_STATIC_PAD_TEMPLATE ("sink",
         "width = (int) 16, "
         "signed = (boolean) true, "
         "channels = (int) 2, "
-        "rate = (int) 44100, " "endianness = (int) byte_order")
-    );
+        "rate = (int) 44100, " "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER)
+    ));
 
 static GstStaticPadTemplate src_temp = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -71,8 +71,8 @@ static GstStaticPadTemplate src_temp = GST_STATIC_PAD_TEMPLATE ("src",
         "width = (int) 16, "
         "signed = (boolean) true, "
         "channels = (int) 2, "
-        "rate = (int) 44100, " "endianness = (int) byte_order")
-    );
+        "rate = (int) 44100, " "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER)
+    ));
 
 enum
 {
