@@ -23,6 +23,7 @@
 #include <gst/gst.h>
 #include "gstpngenc.h"
 #include <gst/video/video.h>
+#include <zlib.h>
 
 #define MAX_HEIGHT		4096
 
@@ -48,7 +49,8 @@ enum
 {
   ARG_0,
   ARG_SNAPSHOT,
-  ARG_NEWMEDIA
+  ARG_NEWMEDIA,
+  ARG_COMPRESSION_LEVEL
 };
 
 static void gst_pngenc_base_init (gpointer g_class);
@@ -61,11 +63,24 @@ static void gst_pngenc_get_property (GObject * object,
 
 static void gst_pngenc_chain (GstPad * pad, GstData * _data);
 
-GstPadTemplate *pngenc_src_template, *pngenc_sink_template;
+static GstStaticPadTemplate pngenc_src_template =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("image/png, "
+        "width = (int) [ 16, 4096 ], "
+        "height = (int) [ 16, 4096 ], " "framerate = (double) [ 0.0, MAX ]")
+    );
+
+static GstStaticPadTemplate pngenc_sink_template =
+    GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_BGRA ";" GST_VIDEO_CAPS_BGR)
+    );
 
 static GstElementClass *parent_class = NULL;
 
-
 static void
 user_error_fn (png_structp png_ptr, png_const_charp error_msg)
 {
@@ -103,39 +118,15 @@ gst_pngenc_get_type (void)
   return pngenc_type;
 }
 
-static GstCaps *
-png_caps_factory (void)
-{
-  return gst_caps_new_simple ("video/x-png",
-      "width", GST_TYPE_INT_RANGE, 16, 4096,
-      "height", GST_TYPE_INT_RANGE, 16, 4096,
-      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
-}
-
-
-static GstCaps *
-raw_caps_factory (void)
-{
-  return gst_caps_from_string (GST_VIDEO_CAPS_RGB);
-}
-
 static void
 gst_pngenc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  GstCaps *raw_caps, *png_caps;
 
-  raw_caps = raw_caps_factory ();
-  png_caps = png_caps_factory ();
-
-  pngenc_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK,
-      GST_PAD_ALWAYS, raw_caps);
-
-  pngenc_src_template = gst_pad_template_new ("src", GST_PAD_SRC,
-      GST_PAD_ALWAYS, png_caps);
-
-  gst_element_class_add_pad_template (element_class, pngenc_sink_template);
-  gst_element_class_add_pad_template (element_class, pngenc_src_template);
+  gst_element_class_add_pad_template
+      (element_class, gst_static_pad_template_get (&pngenc_sink_template));
+  gst_element_class_add_pad_template
+      (element_class, gst_static_pad_template_get (&pngenc_src_template));
   gst_element_class_set_details (element_class, &gst_pngenc_details);
 }
 
@@ -160,6 +151,13 @@ gst_pngenc_class_init (GstPngEncClass * klass)
           "Send new media discontinuity after encoding each frame",
           FALSE, (GParamFlags) G_PARAM_READWRITE));
 
+  g_object_class_install_property
+      (gobject_class, ARG_COMPRESSION_LEVEL,
+      g_param_spec_uint ("compression-level", "compression-level",
+          "PNG compression level",
+          Z_NO_COMPRESSION, Z_BEST_COMPRESSION,
+          6, (GParamFlags) G_PARAM_READWRITE));
+
   gstelement_class->get_property = gst_pngenc_get_property;
   gstelement_class->set_property = gst_pngenc_set_property;
 }
@@ -171,6 +169,8 @@ gst_pngenc_sinklink (GstPad * pad, const GstCaps * caps)
   GstPngEnc *pngenc;
   gdouble fps;
   GstStructure *structure;
+  GstCaps *pcaps;
+  GstPadLinkReturn ret;
 
   pngenc = GST_PNGENC (gst_pad_get_parent (pad));
 
@@ -180,18 +180,21 @@ gst_pngenc_sinklink (GstPad * pad, const GstCaps * caps)
   gst_structure_get_double (structure, "framerate", &fps);
   gst_structure_get_int (structure, "bpp", &pngenc->bpp);
 
-  caps = gst_caps_new_simple ("video/x-png",
+  pcaps = gst_caps_new_simple ("image/png",
       "framerate", G_TYPE_DOUBLE, fps,
       "width", G_TYPE_INT, pngenc->width,
       "height", G_TYPE_INT, pngenc->height, NULL);
 
-  return gst_pad_try_set_caps (pngenc->srcpad, caps);
+  ret = gst_pad_try_set_caps (pngenc->srcpad, pcaps);
+  gst_caps_free (pcaps);
+  return ret;
 }
 
 static void
 gst_pngenc_init (GstPngEnc * pngenc)
 {
-  pngenc->sinkpad = gst_pad_new_from_template (pngenc_sink_template, "sink");
+  pngenc->sinkpad = gst_pad_new_from_template
+      (gst_static_pad_template_get (&pngenc_sink_template), "sink");
   gst_element_add_pad (GST_ELEMENT (pngenc), pngenc->sinkpad);
 
   pngenc->srcpad = gst_pad_new ("src", GST_PAD_SRC);
@@ -205,6 +208,7 @@ gst_pngenc_init (GstPngEnc * pngenc)
 
   pngenc->snapshot = DEFAULT_SNAPSHOT;
   pngenc->newmedia = FALSE;
+  pngenc->compression_level = 6;
 }
 
 static void
@@ -260,33 +264,41 @@ gst_pngenc_chain (GstPad * pad, GstData * _data)
   /* initialize png struct stuff */
   pngenc->png_struct_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING,
       (png_voidp) NULL, user_error_fn, user_warning_fn);
-  /* FIXME: better error handling */
-  if (pngenc->png_struct_ptr == NULL)
-    g_warning ("Failed to initialize png structure");
+  if (pngenc->png_struct_ptr == NULL) {
+    gst_buffer_unref (buf);
+    GST_ELEMENT_ERROR (pngenc, LIBRARY, INIT, (NULL),
+        ("Failed to initialize png structure"));
+    return;
+  }
 
   pngenc->png_info_ptr = png_create_info_struct (pngenc->png_struct_ptr);
   if (!pngenc->png_info_ptr) {
-    png_destroy_read_struct (&(pngenc->png_struct_ptr), (png_infopp) NULL,
-        (png_infopp) NULL);
+    gst_buffer_unref (buf);
+    png_destroy_write_struct (&(pngenc->png_struct_ptr), (png_infopp) NULL);
+    GST_ELEMENT_ERROR (pngenc, LIBRARY, INIT, (NULL),
+        ("Failed to initialize the png info structure"));
+    return;
   }
 
   /* non-0 return is from a longjmp inside of libpng */
   if (setjmp (pngenc->png_struct_ptr->jmpbuf) != 0) {
-    GST_DEBUG ("returning from longjmp");
+    gst_buffer_unref (buf);
     png_destroy_write_struct (&pngenc->png_struct_ptr, &pngenc->png_info_ptr);
+    GST_ELEMENT_ERROR (pngenc, LIBRARY, FAILED, (NULL),
+        ("returning from longjmp"));
     return;
   }
 
   png_set_filter (pngenc->png_struct_ptr, 0,
       PNG_FILTER_NONE | PNG_FILTER_VALUE_NONE);
-  png_set_compression_level (pngenc->png_struct_ptr, 9);
+  png_set_compression_level (pngenc->png_struct_ptr, pngenc->compression_level);
 
   png_set_IHDR (pngenc->png_struct_ptr,
       pngenc->png_info_ptr,
       pngenc->width,
       pngenc->height,
-      pngenc->bpp / 3,
-      PNG_COLOR_TYPE_RGB,
+      8,
+      (pngenc->bpp == 32) ? PNG_COLOR_TYPE_RGBA : PNG_COLOR_TYPE_RGB,
       PNG_INTERLACE_NONE,
       PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 
@@ -295,7 +307,7 @@ gst_pngenc_chain (GstPad * pad, GstData * _data)
 
   for (row_index = 0; row_index < pngenc->height; row_index++)
     row_pointers[row_index] = GST_BUFFER_DATA (buf) +
-        (pngenc->width * row_index * pngenc->bpp / 8);
+        (row_index * pngenc->png_info_ptr->rowbytes);
 
   png_write_info (pngenc->png_struct_ptr, pngenc->png_info_ptr);
   png_write_image (pngenc->png_struct_ptr, row_pointers);
@@ -343,6 +355,9 @@ gst_pngenc_get_property (GObject * object,
     case ARG_NEWMEDIA:
       g_value_set_boolean (value, pngenc->newmedia);
       break;
+    case ARG_COMPRESSION_LEVEL:
+      g_value_set_uint (value, pngenc->compression_level);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -365,6 +380,9 @@ gst_pngenc_set_property (GObject * object,
     case ARG_NEWMEDIA:
       pngenc->newmedia = g_value_get_boolean (value);
       break;
+    case ARG_COMPRESSION_LEVEL:
+      pngenc->compression_level = g_value_get_uint (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
