@@ -752,10 +752,17 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
       break;
       /* the all-formats seek logic */
     case GST_EVENT_SEEK:
-      if (mad->index)
-        res = index_seek (mad, pad, event);
-      else
-        res = normal_seek (mad, pad, event);
+      GST_DEBUG ("forwarding seek event to sink pad");
+      gst_event_ref (event);
+      if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), event)) {
+        /* seek worked, we're done, loop will exit */
+        res = TRUE;
+      } else {
+        if (mad->index)
+          res = index_seek (mad, pad, event);
+        else
+          res = normal_seek (mad, pad, event);
+      }
       break;
 
     default:
@@ -1008,7 +1015,7 @@ is_xhead (unsigned char *buf)
 
 
 #undef LOG
-//#define LOG
+/*#define LOG*/
 #ifdef LOG
 #define lprintf(x...) g_print(x)
 #else
@@ -1025,23 +1032,18 @@ mpg123_parse_xing_header (struct mad_header *header,
   int xflags, xframes, xbytes, xvbr_scale;
   int abr;
   guint8 xtoc[XING_TOC_LENGTH];
-
-  /* This should be the MPEG Audio version ID 
-   * (version 2.5, 2 or 1) least significant byte, but mad doesn't 
-   * provide that, so assume it's always MPEG 1
-   */
-  int lsf_bit = 1;
+  int lsf_bit = !(header->flags & MAD_FLAG_LSF_EXT);
 
   xframes = xbytes = 0;
 
   /* offset of the Xing header */
   if (lsf_bit) {
-    if (header->mode != MAD_MODE_STEREO)
+    if (header->mode != MAD_MODE_SINGLE_CHANNEL)
       ptr += (32 + 4);
     else
       ptr += (17 + 4);
   } else {
-    if (header->mode != MAD_MODE_STEREO)
+    if (header->mode != MAD_MODE_SINGLE_CHANNEL)
       ptr += (17 + 4);
     else
       ptr += (9 + 4);
@@ -1131,7 +1133,7 @@ mpg123_parse_xing_header (struct mad_header *header,
 
 /* internal function to check if the header has changed and thus the
  * caps need to be reset.  Only call during normal mode, not resyncing */
-static void
+static gboolean
 gst_mad_check_caps_reset (GstMad * mad)
 {
   guint nchannels;
@@ -1144,6 +1146,12 @@ gst_mad_check_caps_reset (GstMad * mad)
 #else
   rate = mad->frame.header.samplerate;
 #endif
+  if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE) {
+    GST_INFO_OBJECT (mad,
+        "MAD_OPTION_HALFSAMPLERATE is set, adapting rate from %u to %u", rate,
+        rate >> 1);
+    rate >>= 1;
+  }
 
   /* rate and channels are not supposed to change in a continuous stream,
    * so check this first before doing anything */
@@ -1162,8 +1170,10 @@ gst_mad_check_caps_reset (GstMad * mad)
         mad->pending_channels = nchannels;
         mad->pending_rate = rate;
       }
+      /* Now, we already have a valid caps set and will continue to use
+       * that for a while longer, so we cans afely return TRUE here. */
       if (++mad->times_pending < 3)
-        return;
+        return TRUE;
     }
   }
   gst_mad_update_info (mad);
@@ -1171,9 +1181,6 @@ gst_mad_check_caps_reset (GstMad * mad)
   if (mad->channels != nchannels || mad->rate != rate) {
     GstCaps *caps;
 
-    if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE)
-      rate >>= 1;
-
     /* we set the caps even when the pad is not connected so they
      * can be gotten for streaminfo */
     caps = gst_caps_new_simple ("audio/x-raw-int",
@@ -1183,12 +1190,19 @@ gst_mad_check_caps_reset (GstMad * mad)
         "depth", G_TYPE_INT, 16,
         "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, nchannels, NULL);
 
-    gst_pad_set_explicit_caps (mad->srcpad, caps);
+    if (gst_pad_set_explicit_caps (mad->srcpad, caps)) {
+      mad->caps_set = TRUE;     /* set back to FALSE on discont */
+      mad->channels = nchannels;
+      mad->rate = rate;
+    } else {
+      GST_ELEMENT_ERROR (mad, CORE, NEGOTIATION, (NULL),
+          ("Failed to negotiate %d Hz, %d channels", rate, nchannels));
+      return FALSE;
+    }
     gst_caps_free (caps);
-    mad->caps_set = TRUE;       /* set back to FALSE on discont */
-    mad->channels = nchannels;
-    mad->rate = rate;
   }
+
+  return TRUE;
 }
 
 static void
@@ -1245,7 +1259,15 @@ gst_mad_chain (GstPad * pad, GstData * _data)
     gint tocopy;
     guchar *mad_input_buffer;   /* convenience pointer to tempbuffer */
 
-    tocopy = MIN (MAD_BUFFER_MDLEN, size);
+    tocopy =
+        MIN (MAD_BUFFER_MDLEN, MIN (size,
+            MAD_BUFFER_MDLEN * 3 - mad->tempsize));
+    if (tocopy == 0) {
+      GST_ELEMENT_ERROR (mad, STREAM, DECODE, (NULL),
+          ("mad claims to need more data than %u bytes, we don't have that much",
+              MAD_BUFFER_MDLEN * 3));
+      return;
+    }
 
     /* append the chunk to process to our internal temporary buffer */
     GST_LOG ("tempbuffer size %d, copying %d bytes from incoming buffer",
@@ -1265,17 +1287,29 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       guint nsamples;
       guint64 time_offset;
       guint64 time_duration;
-      unsigned char const *before_sync, *after_sync;
+      gboolean resync = TRUE;
 
       mad->in_error = FALSE;
+
       mad_stream_buffer (&mad->stream, mad_input_buffer, mad->tempsize);
 
+      /* added separate header decoding to catch errors earlier, also fixes
+       * some weird decoding errors... */
+      GST_LOG ("decoding one frame now");
+
       if (mad_frame_decode (&mad->frame, &mad->stream) == -1) {
+        GST_LOG ("got error %d", mad->stream.error);
+
         /* not enough data, need to wait for next buffer? */
         if (mad->stream.error == MAD_ERROR_BUFLEN) {
-          GST_LOG ("not enough data in tempbuffer (%d), breaking to get more",
-              mad->tempsize);
-          break;
+          if (mad->stream.next_frame == mad_input_buffer) {
+            GST_LOG ("not enough data in tempbuffer (%d), breaking to get more",
+                mad->tempsize);
+            break;
+          } else {
+            GST_LOG ("sync error, flushing unneeded data");
+            goto next;
+          }
         }
         /* we are in an error state */
         mad->in_error = TRUE;
@@ -1307,6 +1341,9 @@ gst_mad_chain (GstPad * pad, GstData * _data)
              * id3 tags, so we need to flush one byte less than the tagsize */
             mad_stream_skip (&mad->stream, tagsize - 1);
 
+            /* When we skip, we don't want to call sync */
+            resync = FALSE;
+
             tag = id3_tag_parse (data, tagsize);
             if (tag) {
               GstTagList *list;
@@ -1328,23 +1365,33 @@ gst_mad_chain (GstPad * pad, GstData * _data)
             }
           }
         }
+        //Should not sync here if mad_skip has been used before, the offset
+        //is "pending" inside mad and will be applied on next call to decode.
+        if (resync) {
+          unsigned char const *before_sync, *after_sync;
 
-        mad_frame_mute (&mad->frame);
-        mad_synth_mute (&mad->synth);
-        before_sync = mad->stream.ptr.byte;
-        if (mad_stream_sync (&mad->stream) != 0)
-          GST_WARNING ("mad_stream_sync failed");
-        after_sync = mad->stream.ptr.byte;
-        /* a succesful resync should make us drop bytes as consumed, so
-           calculate from the byte pointers before and after resync */
-        consumed = after_sync - before_sync;
-        GST_DEBUG ("resynchronization consumes %d bytes", consumed);
-        GST_DEBUG ("synced to data: 0x%0x 0x%0x", *mad->stream.ptr.byte,
-            *(mad->stream.ptr.byte + 1));
+          before_sync = mad->stream.ptr.byte;
+          if (mad_stream_sync (&mad->stream) != 0) {
+            consumed = MAD_BUFFER_GUARD < mad->tempsize ?
+                mad->tempsize - MAD_BUFFER_GUARD : 0;
+            GST_DEBUG_OBJECT (mad,
+                "mad_stream_sync failed, skipping all %u bytes we have",
+                consumed);
+          } else {
+            after_sync = mad->stream.ptr.byte;
+            /* a succesful resync should make us drop bytes as consumed, so
+               calculate from the byte pointers before and after resync */
+            consumed = after_sync - before_sync;
+            GST_DEBUG_OBJECT (mad, "resynchronization consumes %d bytes",
+                consumed);
+            GST_DEBUG_OBJECT (mad, "synced to data: 0x%0x 0x%0x",
+                *mad->stream.ptr.byte, *(mad->stream.ptr.byte + 1));
 
+            /* recoverable errors pass */
+          }
+          resync = FALSE;
+        }
 
-        mad_stream_sync (&mad->stream);
-        /* recoverable errors pass */
         goto next;
       }
 
@@ -1373,8 +1420,10 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       }
 
       /* if we're not resyncing/in error, check if caps need to be set again */
-      if (!mad->in_error)
-        gst_mad_check_caps_reset (mad);
+      if (!mad->in_error) {
+        if (!gst_mad_check_caps_reset (mad))
+          goto end;
+      }
       nsamples = MAD_NSBSAMPLES (&mad->frame.header) *
           (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);
 
@@ -1392,8 +1441,9 @@ gst_mad_chain (GstPad * pad, GstData * _data)
               &mad->total_samples);
           mad->last_ts = GST_CLOCK_TIME_NONE;
         }
-        time_offset = mad->total_samples * GST_SECOND / mad->rate;
-        time_duration = (nsamples * GST_SECOND / mad->rate);
+        time_offset =
+            mad->total_samples * GST_SECOND / mad->frame.header.samplerate;
+        time_duration = (nsamples * GST_SECOND / mad->frame.header.samplerate);
       }
 
       if (mad->index) {
@@ -1467,11 +1517,14 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       if (consumed == 0)
         consumed = mad->stream.next_frame - mad_input_buffer;
 
+      if (mad->stream.skiplen > consumed)
+        consumed = mad->stream.skiplen;
       GST_LOG ("mad consumed %d bytes", consumed);
       /* move out pointer to where mad want the next data */
       mad_input_buffer += consumed;
       mad->tempsize -= consumed;
       mad->bytes_consumed += consumed;
+      mad->stream.skiplen = 0;
     }
     /* we only get here from breaks, tempsize never actually drops below 0 */
     memmove (mad->tempbuffer, mad_input_buffer, mad->tempsize);
