@@ -22,10 +22,9 @@
 #include <config.h>
 #endif
 #include <gst/gst.h>
-#include "gsttextrender.h"
 #include "gsttextoverlay.h"
 
-GST_DEBUG_CATEGORY (pango_debug);
+GST_DEBUG_CATEGORY_STATIC (pango_debug);
 #define GST_CAT_DEFAULT pango_debug
 
 static GstElementDetails textoverlay_details = {
@@ -218,7 +217,7 @@ render_text (GstTextOverlay * overlay)
 
   pango_layout_get_pixel_extents (overlay->layout, &ink_rect, &logical_rect);
   resize_bitmap (overlay, ink_rect.width, ink_rect.height + ink_rect.y);
-  pango_ft2_render_layout (&overlay->bitmap, overlay->layout, -ink_rect.x, 0);
+  pango_ft2_render_layout (&overlay->bitmap, overlay->layout, 0, 0);
   overlay->baseline_y = ink_rect.y;
 }
 
@@ -296,16 +295,6 @@ gst_textoverlay_event (GstPad * pad, GstEvent * event)
       GST_PAD_IS_LINKED (overlay->text_sinkpad)) {
     gst_event_ref (event);
     gst_pad_send_event (GST_PAD_PEER (overlay->text_sinkpad), event);
-    if (overlay->next_data) {
-      gst_data_unref (overlay->next_data);
-      overlay->next_data = NULL;
-    }
-    if (overlay->current_data) {
-      gst_data_unref (overlay->current_data);
-      overlay->current_data = NULL;
-    }
-    overlay->need_render = TRUE;
-    overlay->text_eos = FALSE;
   }
 
   return gst_pad_send_event (GST_PAD_PEER (overlay->video_sinkpad), event);
@@ -320,7 +309,7 @@ gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
   int rowinc, bit_rowinc, uv_rowinc;
   guchar *p, *bitp, *u_p;
   int video_width = overlay->width, video_height = overlay->height;
-  int bitmap_x0 = 0;            //x0 < 1 ? -(x0 - 1) : 1;       /* 1 pixel border */
+  int bitmap_x0 = x0 < 1 ? -(x0 - 1) : 1;       /* 1 pixel border */
   int bitmap_y0 = y0 < 1 ? -(y0 - 1) : 1;       /* 1 pixel border */
   int bitmap_width = bitmap->width - bitmap_x0;
   int bitmap_height = bitmap->rows - bitmap_y0;
@@ -342,7 +331,7 @@ gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
   x1 = x0 + bitmap_x0;
   p = pixbuf + video_width * y1 + x1;
   bitp = bitmap->buffer + bitmap->pitch * bitmap_y0 + bitmap_x0;
-  for (y = bitmap_y0; y < bitmap_y0 + bitmap_height; y++) {
+  for (y = bitmap_y0; y < bitmap_height; y++) {
     int n;
 
     for (n = bitmap_width; n > 0; --n) {
@@ -371,7 +360,7 @@ gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
   skip_y = 0;
   skip_x = 0;
 
-  for (; y < bitmap_y0 + bitmap_height; y++) {
+  for (; y < bitmap_height; y++) {
     int n;
 
     x1 = x0 + bitmap_x0;
@@ -482,25 +471,22 @@ gst_textoverlay_loop (GstElement * element)
       GstEvent *event = GST_EVENT (video_frame);
       GstEventType type = GST_EVENT_TYPE (event);
 
-      GST_DEBUG ("Received event of type %d", type);
-      if (type == GST_EVENT_INTERRUPT) {
-        gst_event_unref (event);
-        return;
-      }
       gst_pad_event_default (overlay->video_sinkpad, event);
-      if (type == GST_EVENT_EOS) {
-        if (!overlay->text_eos) {
-          /* EOS text stream */
-          GstData *data = NULL;
+      GST_DEBUG ("Received event of type %d", type);
+      if (type == GST_EVENT_INTERRUPT)
+        return;
+      else if (type == GST_EVENT_EOS) {
+        /* EOS text stream */
+        GstData *data = NULL;
+
+        do {
+          if (data)
+            gst_data_unref (data);
+          data = gst_pad_pull (overlay->text_sinkpad);
+        } while (!GST_IS_EVENT (data) ||
+            GST_EVENT_TYPE (data) == GST_EVENT_EOS);
+        gst_data_unref (data);
 
-          do {
-            if (data)
-              gst_data_unref (data);
-            data = gst_pad_pull (overlay->text_sinkpad);
-          } while (!GST_IS_EVENT (data) ||
-              GST_EVENT_TYPE (data) == GST_EVENT_EOS);
-          gst_data_unref (data);
-        }
         return;
       }
       video_frame = NULL;
@@ -526,7 +512,7 @@ gst_textoverlay_loop (GstElement * element)
     GST_DEBUG ("attempting to pull text data");
 
     /* read all text buffers until we get one "in the future" */
-    if (!GST_PAD_IS_USABLE (overlay->text_sinkpad) || overlay->text_eos) {
+    if (!GST_PAD_IS_USABLE (overlay->text_sinkpad)) {
       break;
     }
     do {
@@ -536,19 +522,15 @@ gst_textoverlay_loop (GstElement * element)
         GstEvent *event = GST_EVENT (overlay->next_data);
         GstEventType type = GST_EVENT_TYPE (event);
 
-        GST_DEBUG ("Received text data event of type %d", type);
-        overlay->next_data = NULL;
         gst_event_unref (event);
-        if (type == GST_EVENT_EOS) {
-          overlay->text_eos = TRUE;
+        if (type == GST_EVENT_EOS)
           break;
-        } else if (type == GST_EVENT_INTERRUPT)
+        else if (type == GST_EVENT_INTERRUPT)
           return;
+        overlay->next_data = NULL;
       }
     } while (!overlay->next_data);
 
-    if (!overlay->next_data)
-      break;
     if (PAST_END (overlay->next_data, now)) {
       GST_DEBUG ("Received %s is past end (%" GST_TIME_FORMAT " + %"
           GST_TIME_FORMAT " < %" GST_TIME_FORMAT ")",
@@ -592,7 +574,7 @@ gst_textoverlay_loop (GstElement * element)
           GST_BUFFER_DATA (overlay->current_data));
       /* somehow pango barfs over "\0" buffers... */
       pango_layout_set_markup (overlay->layout,
-          (gchar *) GST_BUFFER_DATA (overlay->current_data), size);
+          GST_BUFFER_DATA (overlay->current_data), size);
     } else {
       GST_DEBUG ("Filler - no data");
       pango_layout_set_markup (overlay->layout, "", 0);
@@ -637,11 +619,12 @@ gst_textoverlay_change_state (GstElement * element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
-      overlay->text_eos = FALSE;
       break;
   }
 
-  return parent_class->change_state (element);
+  parent_class->change_state (element);
+
+  return GST_STATE_SUCCESS;
 }
 
 static void
@@ -709,7 +692,6 @@ gst_textoverlay_init (GstTextOverlay * overlay)
 
   overlay->default_text = g_strdup ("");
   overlay->need_render = TRUE;
-  overlay->text_eos = FALSE;
 
   gst_element_set_loop_function (GST_ELEMENT (overlay), gst_textoverlay_loop);
 
@@ -811,9 +793,7 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
   if (!gst_element_register (plugin, "textoverlay", GST_RANK_NONE,
-          GST_TYPE_TEXTOVERLAY) ||
-      !gst_element_register (plugin, "textrender", GST_RANK_NONE,
-          GST_TYPE_TEXT_RENDER))
+          GST_TYPE_TEXTOVERLAY))
     return FALSE;
 
   /*texttestsrc_plugin_init(module, plugin); */
