@@ -170,6 +170,7 @@ gst_speex_dec_init (GstSpeexDec * dec)
       (&speex_dec_sink_factory), "sink");
   gst_pad_set_chain_function (dec->sinkpad, speex_dec_chain);
   gst_pad_set_formats_function (dec->sinkpad, speex_dec_get_formats);
+  gst_pad_set_convert_function (dec->sinkpad, speex_dec_convert);
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
 
   dec->srcpad =
@@ -203,6 +204,10 @@ speex_dec_convert (GstPad * pad,
   if (dec->packetno < 1)
     return FALSE;
 
+  if (pad == dec->sinkpad &&
+      (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
+    return FALSE;
+
   switch (src_format) {
     case GST_FORMAT_TIME:
       switch (*dest_format) {
@@ -453,6 +458,8 @@ speex_dec_chain (GstPad * pad, GstData * data)
           GST_TAG_ENCODER, encoder, NULL);
       g_free (encoder);
     }
+    gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+        GST_TAG_AUDIO_CODEC, "Speex", NULL);
     /*
        gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
        GST_TAG_ENCODER_VERSION, dec->vi.version, NULL);
