@@ -23,7 +23,7 @@
 #endif
 
 #include <gst/gstutils.h>
-#include <gst/riff/riff-ids.h>
+#include <gst/riff/riff-media.h>
 #include <string.h>
 
 #include "gstasfdemux.h"
@@ -69,10 +69,6 @@ static gboolean gst_asf_demux_setup_pad (GstASFDemux * asf_demux,
     GstPad * src_pad, GstCaps * caps, guint16 id);
 
 static GstElementStateReturn gst_asf_demux_change_state (GstElement * element);
-static GstCaps *gst_asf_demux_video_caps (guint32 codec_fcc,
-    asf_stream_video_format * video, char **codec_name);
-static GstCaps *gst_asf_demux_audio_caps (guint16 codec_id,
-    asf_stream_audio * audio, guint8 * extradata, char **codec_name);
 
 static GstPadTemplate *videosrctempl, *audiosrctempl;
 static GstElementClass *parent_class = NULL;
@@ -114,52 +110,14 @@ gst_asf_demux_base_init (gpointer g_class)
     "Demultiplexes ASF Streams",
     "Owen Fraser-Green <owen@discobabe.net>"
   };
-  int i;
-  GstCaps *audcaps, *vidcaps, *temp;
-  guint32 vid_list[] = {
-    GST_MAKE_FOURCC ('I', '4', '2', '0'),
-    GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'),
-    GST_MAKE_FOURCC ('M', 'J', 'P', 'G'),
-    GST_MAKE_FOURCC ('D', 'V', 'S', 'D'),
-    GST_MAKE_FOURCC ('W', 'M', 'V', '1'),
-    GST_MAKE_FOURCC ('W', 'M', 'V', '2'),
-    GST_MAKE_FOURCC ('W', 'M', 'V', '3'),
-    GST_MAKE_FOURCC ('M', 'P', 'G', '4'),
-    GST_MAKE_FOURCC ('M', 'P', '4', '2'),
-    GST_MAKE_FOURCC ('M', 'P', '4', '3'),
-    GST_MAKE_FOURCC ('D', 'I', 'V', '3'),
-    GST_MAKE_FOURCC ('D', 'X', '5', '0'),
-    0                           /* end */
-  };
-  gint aud_list[] = {
-    GST_RIFF_WAVE_FORMAT_MPEGL3,
-    GST_RIFF_WAVE_FORMAT_MPEGL12,
-    GST_RIFF_WAVE_FORMAT_PCM,
-    GST_RIFF_WAVE_FORMAT_VORBIS1,
-    GST_RIFF_WAVE_FORMAT_A52,
-    GST_RIFF_WAVE_FORMAT_WMAV1,
-    GST_RIFF_WAVE_FORMAT_WMAV2,
-    GST_RIFF_WAVE_FORMAT_WMAV3,
-    -1                          /* end */
-  };
-
-  audcaps = gst_caps_new_empty ();
-  for (i = 0; aud_list[i] != -1; i++) {
-    temp = gst_asf_demux_audio_caps (aud_list[i], NULL, NULL, NULL);
-    gst_caps_append (audcaps, temp);
-  }
+  GstCaps *audcaps = gst_riff_create_audio_template_caps (),
+      *vidcaps = gst_riff_create_video_template_caps ();
 
   audiosrctempl = gst_pad_template_new ("audio_%02d",
       GST_PAD_SRC, GST_PAD_SOMETIMES, audcaps);
-
-  vidcaps = gst_caps_new_empty ();
-  for (i = 0; vid_list[i] != 0; i++) {
-    temp = gst_asf_demux_video_caps (vid_list[i], NULL, NULL);
-    gst_caps_append (vidcaps, temp);
-  }
-
   videosrctempl = gst_pad_template_new ("video_%02d",
       GST_PAD_SRC, GST_PAD_SOMETIMES, vidcaps);
+
   gst_element_class_add_pad_template (element_class, audiosrctempl);
   gst_element_class_add_pad_template (element_class, videosrctempl);
   gst_element_class_add_pad_template (element_class,
@@ -211,6 +169,7 @@ gst_asf_demux_init (GstASFDemux * asf_demux)
   asf_demux->num_video_streams = 0;
   asf_demux->num_streams = 0;
 
+  asf_demux->taglist = NULL;
   asf_demux->state = GST_ASF_DEMUX_STATE_HEADER;
   asf_demux->seek_pending = GST_CLOCK_TIME_NONE;
   asf_demux->seek_discont = FALSE;
@@ -260,9 +219,38 @@ gst_asf_demux_loop (GstElement * element)
     case GST_ASF_DEMUX_STATE_HEADER:
       gst_asf_demux_process_object (asf_demux);
       break;
-    case GST_ASF_DEMUX_STATE_DATA:
+    case GST_ASF_DEMUX_STATE_DATA:{
+      guint64 start_off = gst_bytestream_tell (asf_demux->bs), end_off;
+
+      /* make sure a full packet is actually available */
+      if (asf_demux->packet_size != (guint32) - 1)
+        do {
+          guint32 remaining;
+          GstEvent *event;
+          guint got_bytes;
+          guint8 *data;
+
+          if ((got_bytes = gst_bytestream_peek_bytes (asf_demux->bs, &data,
+                      asf_demux->packet_size)) == asf_demux->packet_size)
+            break;
+          gst_bytestream_get_status (asf_demux->bs, &remaining, &event);
+
+          if (!gst_asf_demux_handle_sink_event (asf_demux, event, remaining))
+            return;
+        } while (1);
+
+      /* now handle data */
       gst_asf_demux_handle_data (asf_demux);
+
+      /* align by packet size */
+      end_off = gst_bytestream_tell (asf_demux->bs);
+      if (asf_demux->packet_size != (guint32) - 1 &&
+          end_off - start_off < asf_demux->packet_size) {
+        gst_bytestream_flush_fast (asf_demux->bs,
+            asf_demux->packet_size - (end_off - start_off));
+      }
       break;
+    }
     case GST_ASF_DEMUX_STATE_EOS:
       gst_pad_event_default (asf_demux->sinkpad, gst_event_new (GST_EVENT_EOS));
       break;
@@ -286,15 +274,17 @@ _read_var_length (GstASFDemux * asf_demux, guint8 type, guint32 * rsize)
 
   got_bytes = gst_bytestream_peek_bytes (bs, &var, 4);
 
-  while (got_bytes < 4) {
+  do {
     guint32 remaining;
     GstEvent *event;
 
+    if ((got_bytes = gst_bytestream_peek_bytes (bs, &var, 4)) == 4)
+      break;
     gst_bytestream_get_status (bs, &remaining, &event);
-    gst_event_unref (event);
 
-    got_bytes = gst_bytestream_peek_bytes (bs, &var, 4);
-  }
+    if (!gst_asf_demux_handle_sink_event (asf_demux, event, remaining))
+      return FALSE;
+  } while (1);
 
   switch (type) {
     case 1:
@@ -340,12 +330,13 @@ _read_uint ## bits (GstASFDemux *asf_demux, guint ## bits *ret)	        	\
 }
 
 #define GST_READ_UINT8_LE(x) GST_READ_UINT8(x)
-READ_UINT_BITS_FUNCTION (8)
-    READ_UINT_BITS_FUNCTION (16)
-    READ_UINT_BITS_FUNCTION (32)
-    READ_UINT_BITS_FUNCTION (64)
-#define GET_UINT(a,b)
-     static gboolean _read_guid (GstASFDemux * asf_demux, ASFGuid * guid)
+READ_UINT_BITS_FUNCTION (8);
+READ_UINT_BITS_FUNCTION (16);
+READ_UINT_BITS_FUNCTION (32);
+READ_UINT_BITS_FUNCTION (64);
+
+static gboolean
+_read_guid (GstASFDemux * asf_demux, ASFGuid * guid)
 {
   return (_read_uint32 (asf_demux, &guid->v1) &&
       _read_uint32 (asf_demux, &guid->v2) &&
@@ -542,6 +533,40 @@ gst_asf_demux_process_bitrate_props_object (GstASFDemux * asf_demux,
   return TRUE;
 }
 
+static void
+gst_asf_demux_commit_taglist (GstASFDemux * asf_demux, GstTagList * taglist)
+{
+  GstElement *element = GST_ELEMENT (asf_demux);
+  GstEvent *event;
+  const GList *padlist;
+
+  /* emit signal */
+  gst_element_found_tags (element, taglist);
+
+  /* save internally */
+  if (!asf_demux->taglist)
+    asf_demux->taglist = gst_tag_list_copy (taglist);
+  else {
+    GstTagList *t;
+
+    t = gst_tag_list_merge (asf_demux->taglist, taglist, GST_TAG_MERGE_APPEND);
+    gst_tag_list_free (asf_demux->taglist);
+    asf_demux->taglist = t;
+  }
+
+  /* pads */
+  event = gst_event_new_tag (taglist);
+  for (padlist = gst_element_get_pad_list (element);
+      padlist != NULL; padlist = padlist->next) {
+    if (GST_PAD_IS_SRC (padlist->data) && GST_PAD_IS_USABLE (padlist->data)) {
+      gst_event_ref (event);
+      gst_pad_push (GST_PAD (padlist->data), GST_DATA (event));
+    }
+  }
+  gst_event_unref (event);
+}
+
+/* Content Description Object */
 static gboolean
 gst_asf_demux_process_comment (GstASFDemux * asf_demux, guint64 * obj_size)
 {
@@ -599,22 +624,7 @@ gst_asf_demux_process_comment (GstASFDemux * asf_demux, guint64 * obj_size)
   g_value_unset (&value);
 
   if (have_tags) {
-    GstElement *element = GST_ELEMENT (asf_demux);
-    GstEvent *event;
-    const GList *padlist;
-
-    gst_element_found_tags (element, taglist);
-    event = gst_event_new_tag (taglist);
-
-    for (padlist = gst_element_get_pad_list (element);
-        padlist != NULL; padlist = padlist->next) {
-      if (GST_PAD_IS_SRC (padlist->data) && GST_PAD_IS_USABLE (padlist->data)) {
-        gst_event_ref (event);
-        gst_pad_push (GST_PAD (padlist->data), GST_DATA (event));
-      }
-    }
-
-    gst_event_unref (event);
+    gst_asf_demux_commit_taglist (asf_demux, taglist);
   } else {
     gst_tag_list_free (taglist);
   }
@@ -627,6 +637,156 @@ fail:
   return FALSE;
 }
 
+/* Extended Content Description Object */
+static gboolean
+gst_asf_demux_process_ext_content_desc (GstASFDemux * asf_demux,
+    guint64 * obj_size)
+{
+  GstByteStream *bs = asf_demux->bs;
+  guint16 blockcount, i;
+
+/* 
+
+Other known (and unused) 'text/unicode' metadata available :
+
+ WM/Lyrics =
+ WM/MediaPrimaryClassID = {D1607DBC-E323-4BE2-86A1-48A42A28441E}
+ WMFSDKVersion = 9.00.00.2980
+ WMFSDKNeeded = 0.0.0.0000
+ WM/UniqueFileIdentifier = AMGa_id=R    15334;AMGp_id=P     5149;AMGt_id=T  2324984
+ WM/Publisher = 4AD
+ WM/Provider = AMG
+ WM/ProviderRating = 8
+ WM/ProviderStyle = Rock (similar to WM/Genre)
+ WM/GenreID (similar to WM/Genre)
+
+Other known (and unused) 'non-text' metadata available :
+
+WM/Track (same as WM/TrackNumber but starts at 0)
+WM/EncodingTime
+WM/MCDI
+IsVBR
+
+*/
+
+  const guchar *tags[] = { GST_TAG_GENRE, GST_TAG_ALBUM, GST_TAG_ARTIST, GST_TAG_TRACK_NUMBER, GST_TAG_DATE, NULL };    // GST_TAG_COMPOSER
+  const guchar *tags_label[] = { "WM/Genre", "WM/AlbumTitle", "WM/AlbumArtist", "WM/TrackNumber", "WM/Year", NULL };    // "WM/Composer"
+
+  GstTagList *taglist;
+  GValue tag_value = { 0 };
+  gboolean have_tags = FALSE;
+  guint8 *name = NULL;
+
+  GST_INFO ("Object is an extended content description.");
+
+  taglist = gst_tag_list_new ();
+
+  /* Content Descriptors Count */
+  if (!_read_uint16 (asf_demux, &blockcount))
+    goto fail;
+
+  for (i = 1; i <= blockcount; i++) {
+    guint16 name_length;
+    guint16 datatype;
+    guint16 value_length;
+    guint8 *tmpname;
+    guint8 *tmpvalue, *value;
+
+    /* Descriptor Name Length */
+    if (!_read_uint16 (asf_demux, &name_length))
+      goto fail;
+
+    /* Descriptor Name */
+    name = g_malloc (name_length);
+    if (gst_bytestream_peek_bytes (bs, &tmpname, name_length) != name_length)
+      goto fail;
+
+    name = g_memdup (tmpname, name_length);
+    gst_bytestream_flush_fast (bs, name_length);
+
+    /* Descriptor Value Data Type */
+    if (!_read_uint16 (asf_demux, &datatype))
+      goto fail;
+
+    /* Descriptor Value Length */
+    if (!_read_uint16 (asf_demux, &value_length))
+      goto fail;
+
+    /* Descriptor Value */
+    if (gst_bytestream_peek_bytes (bs, &tmpvalue, value_length) != value_length)
+      goto fail;
+
+    value = g_memdup (tmpvalue, value_length);
+    gst_bytestream_flush_fast (bs, value_length);
+
+    /* 0000 = Unicode String   OR   0003 = DWORD */
+    if ((datatype == 0) || (datatype == 3)) {
+      gsize in, out;
+      guchar tag;
+      guint16 j = 0;
+
+      /* convert to UTF-8 */
+      name =
+          g_convert (name, name_length, "UTF-8", "UTF-16LE", &in, &out, NULL);
+      name[out] = 0;
+
+      while (tags_label[j] != NULL) {
+        if (!strcmp (name, tags_label[j])) {
+          tag = j;
+
+          /* 0000 = UTF-16 String */
+          if (datatype == 0) {
+            /* convert to UTF-8 */
+            value =
+                g_convert (value, value_length, "UTF-8", "UTF-16LE", &in, &out,
+                NULL);
+            value[out] = 0;
+
+            /* get rid of tags with empty value */
+            if (strlen (value)) {
+              g_value_init (&tag_value, G_TYPE_STRING);
+              g_value_set_string (&tag_value, value);
+            }
+          }
+
+          /* 0003 = DWORD */
+          if (datatype == 3) {
+            g_value_init (&tag_value, G_TYPE_INT);
+            g_value_set_int (&tag_value, GUINT32_FROM_LE ((guint32) * value));
+          }
+
+          if (G_IS_VALUE (&tag_value)) {
+            gst_tag_list_add_values (taglist, GST_TAG_MERGE_APPEND, tags[tag],
+                &tag_value, NULL);
+
+            g_value_unset (&tag_value);
+
+            have_tags = TRUE;
+          }
+        }
+        j++;
+      }
+    }
+
+    g_free (name);
+    g_free (value);
+  }
+
+  if (have_tags) {
+    gst_asf_demux_commit_taglist (asf_demux, taglist);
+  } else {
+    gst_tag_list_free (taglist);
+  }
+
+  return TRUE;
+
+fail:
+  if (name) {
+    g_free (name);
+  };
+  return FALSE;
+}
+
 static gboolean
 gst_asf_demux_process_header (GstASFDemux * asf_demux, guint64 * obj_size)
 {
@@ -764,7 +924,14 @@ gst_asf_demux_process_segment (GstASFDemux * asf_demux,
       if (!gst_asf_demux_process_chunk (asf_demux, packet_info, &segment_info))
         return FALSE;
 
-      frag_size -= segment_info.chunk_size + 1;
+      if (segment_info.chunk_size < frag_size)
+        frag_size -= segment_info.chunk_size + 1;
+      else {
+        GST_ELEMENT_ERROR (asf_demux, STREAM, TOO_LAZY,
+            ("Invalid data in stream"),
+            ("Invalid fragment size indicator in segment"));
+        return FALSE;
+      }
     }
   } else {
     segment_info.chunk_size = frag_size;
@@ -875,13 +1042,13 @@ gst_asf_demux_handle_data (GstASFDemux * asf_demux)
   flags = packet_properties_object.flags;
   property = packet_properties_object.property;
 
+  packet_length = _read_var_length (asf_demux, (flags >> 5) & 0x03, &rsize);
+  if (packet_length == 0)
+    packet_length = asf_demux->packet_size;
   packet_info.multiple = flags & 0x01;
   sequence = _read_var_length (asf_demux, (flags >> 1) & 0x03, &rsize);
   packet_info.padsize =
       _read_var_length (asf_demux, (flags >> 3) & 0x03, &rsize);
-  packet_length = _read_var_length (asf_demux, (flags >> 5) & 0x03, &rsize);
-  if (packet_length == 0)
-    packet_length = asf_demux->packet_size;
 
   GST_DEBUG ("Multiple = %u, Sequence = %u, Padsize = %u, Packet length = %u",
       packet_info.multiple, sequence, packet_info.padsize, packet_length);
@@ -917,7 +1084,6 @@ gst_asf_demux_handle_data (GstASFDemux * asf_demux)
     if (!gst_asf_demux_process_segment (asf_demux, &packet_info))
       return FALSE;
   }
-
   /* Skip the padding */
   if (packet_info.padsize > 0)
     gst_bytestream_flush (asf_demux->bs, packet_info.padsize);
@@ -942,8 +1108,6 @@ gst_asf_demux_process_stream (GstASFDemux * asf_demux, guint64 * obj_size)
   asf_stream_video video_object;
   asf_stream_video_format video_format_object;
   guint16 size;
-  GstBuffer *buf;
-  guint32 got_bytes;
   guint16 id;
 
   /* Get the rest of the header's header */
@@ -1018,21 +1182,6 @@ gst_asf_demux_process_stream (GstASFDemux * asf_demux, guint64 * obj_size)
 
       if (!gst_asf_demux_add_video_stream (asf_demux, &video_format_object, id))
         return FALSE;
-
-      /* Read any additional information */
-      if (size) {
-        got_bytes = gst_bytestream_read (asf_demux->bs, &buf, size);
-        /* There is additional data */
-        while (got_bytes < size) {
-          guint32 remaining;
-          GstEvent *event;
-
-          gst_bytestream_get_status (asf_demux->bs, &remaining, &event);
-          gst_event_unref (event);
-
-          got_bytes = gst_bytestream_read (asf_demux->bs, &buf, size);
-        }
-      }
       break;
     default:
       GST_ELEMENT_ERROR (asf_demux, STREAM, WRONG_TYPE, (NULL),
@@ -1113,13 +1262,14 @@ gst_asf_demux_process_object (GstASFDemux * asf_demux)
       return gst_asf_demux_process_header_ext (asf_demux, &obj_size);
     case ASF_OBJ_BITRATE_PROPS:
       return gst_asf_demux_process_bitrate_props_object (asf_demux, &obj_size);
+    case ASF_OBJ_EXT_CONTENT_DESC:
+      return gst_asf_demux_process_ext_content_desc (asf_demux, &obj_size);
     case ASF_OBJ_CONCEAL_NONE:
     case ASF_OBJ_HEAD2:
     case ASF_OBJ_UNDEFINED:
     case ASF_OBJ_CODEC_COMMENT:
     case ASF_OBJ_INDEX:
     case ASF_OBJ_PADDING:
-    case ASF_OBJ_EXT_CONTENT_DESC:
     case ASF_OBJ_BITRATE_MUTEX:
     case ASF_OBJ_LANGUAGE_LIST:
     case ASF_OBJ_METADATA_OBJECT:
@@ -1220,6 +1370,8 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
     stream->sequence = segment_info->sequence;
     asf_demux->pts = segment_info->frag_timestamp - asf_demux->preroll;
     got_bytes = gst_bytestream_peek (bs, &buffer, segment_info->chunk_size);
+    if (got_bytes == 0)
+      goto done;
     GST_DEBUG ("BUFFER: Copied stream to buffer (%p - %d)", buffer,
         GST_BUFFER_REFCOUNT_VALUE (buffer));
     stream->payload = buffer;
@@ -1235,6 +1387,8 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
       /* continuing packet */
       GST_INFO ("A continuation packet");
       got_bytes = gst_bytestream_peek (bs, &buffer, segment_info->chunk_size);
+      if (got_bytes == 0)
+        goto done;
       GST_DEBUG ("Copied stream to buffer (%p - %d)", buffer,
           GST_BUFFER_REFCOUNT_VALUE (buffer));
       new_buffer = gst_buffer_merge (stream->payload, buffer);
@@ -1273,6 +1427,11 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
 
   if (stream->frag_offset < segment_info->segment_size) {
     /* We don't have the whole packet yet */
+  } else if (got_bytes < segment_info->chunk_size) {
+    /* We didn't get the entire chunk, so don't push it down the pipeline. */
+    gst_buffer_unref (stream->payload);
+    stream->payload = NULL;
+    stream->frag_offset = 0;
   } else {
     /* We have the whole packet now so we should push the packet to
        the src pad now. First though we should check if we need to do
@@ -1308,6 +1467,9 @@ gst_asf_demux_process_chunk (GstASFDemux * asf_demux,
       }
 
       gst_pad_push (stream->pad, GST_DATA (stream->payload));
+    } else {
+      gst_buffer_unref (stream->payload);
+      stream->payload = NULL;
     }
 
     stream->frag_offset = 0;
@@ -1366,6 +1528,7 @@ gst_asf_demux_handle_sink_event (GstASFDemux * asf_demux,
     default:
       GST_WARNING_OBJECT (asf_demux, "unhandled event %d",
           GST_EVENT_TYPE (event));
+      ret = FALSE;
       break;
   }
 
@@ -1524,6 +1687,10 @@ gst_asf_demux_change_state (GstElement * element)
       for (i = 0; i < GST_ASF_DEMUX_NUM_AUDIO_PADS; i++) {
         asf_demux->audio_PTS[i] = 0;
       }
+      if (asf_demux->taglist) {
+        gst_tag_list_free (asf_demux->taglist);
+        asf_demux->taglist = NULL;
+      }
       asf_demux->state = GST_ASF_DEMUX_STATE_HEADER;
       asf_demux->seek_pending = GST_CLOCK_TIME_NONE;
       asf_demux->seek_discont = FALSE;
@@ -1559,135 +1726,6 @@ gst_asf_demux_identify_guid (GstASFDemux * asf_demux,
   return ASF_OBJ_UNDEFINED;
 }
 
-
-static GstCaps *
-gst_asf_demux_audio_caps (guint16 codec_id,
-    asf_stream_audio * audio, guint8 * extradata, char **codec_name)
-{
-  GstCaps *caps;
-  gint flags1, flags2;
-
-  flags1 = 0;
-  flags2 = 0;
-
-  switch (codec_id) {
-    case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
-      caps = gst_caps_from_string ("audio/mpeg, mpegversion = (int) 1, "
-          "layer = (int) 3");
-      if (codec_name)
-        *codec_name = g_strdup ("MPEG 1 layer 3");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
-      caps = gst_caps_from_string ("audio/mpeg, mpegversion = (int) 1, "
-          "layer = (int) 2");
-      if (codec_name)
-        *codec_name = g_strdup ("MPEG 1 layer 2");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_PCM:     /* PCM/wav */  {
-      caps = gst_caps_from_string ("audio/x-raw-int, "
-          "endianness = (int) LITTLE_ENDIAN,"
-          "signed = (boolean) { true, false }, "
-          "width = (int) { 8, 16 }, " "depth = (int) { 8, 16 }");
-
-      if (audio != NULL) {
-        gint ba = audio->block_align;
-        gint ch = audio->channels;
-        gint ws = audio->word_size;
-
-        gst_caps_set_simple (caps,
-            "width", G_TYPE_INT, (int) (ba * 8 / ch),
-            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, (ws != 8), NULL);
-      }
-      if (codec_name)
-        *codec_name = g_strdup ("PCM WAV");
-    }
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_VORBIS1: /* vorbis mode 1 */
-    case GST_RIFF_WAVE_FORMAT_VORBIS2: /* vorbis mode 2 */
-    case GST_RIFF_WAVE_FORMAT_VORBIS3: /* vorbis mode 3 */
-    case GST_RIFF_WAVE_FORMAT_VORBIS1PLUS:     /* vorbis mode 1+ */
-    case GST_RIFF_WAVE_FORMAT_VORBIS2PLUS:     /* vorbis mode 2+ */
-    case GST_RIFF_WAVE_FORMAT_VORBIS3PLUS:     /* vorbis mode 3+ */
-      caps = gst_caps_from_string ("audio/x-vorbis");
-      if (codec_name)
-        *codec_name = g_strdup ("Vorbis");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_A52:
-      caps = gst_caps_from_string ("audio/x-ac3");
-      if (codec_name)
-        *codec_name = g_strdup ("AC3");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_WMAV1:
-      caps = gst_caps_from_string ("audio/x-wma, "
-          "wmaversion = (int) 1, "
-          "block_align = (int) [ 0, MAX ], " "bitrate = (int) [ 0, MAX ]");
-      if (audio != NULL) {
-        GstBuffer *buffer;
-
-        buffer = gst_buffer_new_and_alloc (audio->size);
-        memcpy (GST_BUFFER_DATA (buffer), extradata, audio->size);
-        /* gst_util_dump_mem (GST_BUFFER_DATA (buffer), audio->size); */
-
-        gst_caps_set_simple (caps,
-            "codec_data", GST_TYPE_BUFFER, buffer,
-            "block_align", G_TYPE_INT, audio->block_align,
-            "bitrate", G_TYPE_INT, audio->byte_rate * 8, NULL);
-      }
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft Windows Media 7 (WMA1)");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_WMAV2:
-      caps = gst_caps_from_string ("audio/x-wma, "
-          "wmaversion = (int) 2, "
-          "block_align = (int) [ 0, MAX ], " "bitrate = (int) [ 0, MAX ]");
-      if (audio != NULL) {
-        GstBuffer *buffer;
-
-        buffer = gst_buffer_new_and_alloc (audio->size);
-        memcpy (GST_BUFFER_DATA (buffer), extradata, audio->size);
-        /* gst_util_dump_mem (GST_BUFFER_DATA (buffer), audio->size); */
-
-        gst_caps_set_simple (caps,
-            "codec_data", GST_TYPE_BUFFER, buffer,
-            "block_align", G_TYPE_INT, audio->block_align,
-            "bitrate", G_TYPE_INT, audio->byte_rate * 8, NULL);
-        gst_data_unref (GST_DATA (buffer));
-      }
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft Windows Media 8 (WMA2)");
-      break;
-
-    case GST_RIFF_WAVE_FORMAT_WMAV3:
-      caps = gst_caps_from_string ("audio/x-wma, " "wmaversion = (int) 3");
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft Windows Media 9 (WMA3)");
-      break;
-
-    default:
-      GST_WARNING ("asfdemux: unknown audio format 0x%04x", codec_id);
-      return GST_CAPS_ANY;
-      break;
-  }
-
-  if (audio != NULL) {
-    gst_caps_set_simple (caps,
-        "rate", G_TYPE_INT, audio->sample_rate,
-        "channels", G_TYPE_INT, audio->channels, NULL);
-  } else {
-    gst_caps_set_simple (caps,
-        "rate", GST_TYPE_INT_RANGE, 8000, 96000,
-        "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
-  }
-
-  return caps;
-}
-
 static gboolean
 gst_asf_demux_add_audio_stream (GstASFDemux * asf_demux,
     asf_stream_audio * audio, guint16 id)
@@ -1698,6 +1736,7 @@ gst_asf_demux_add_audio_stream (GstASFDemux * asf_demux,
   guint16 size_left = 0;
   char *codec_name = NULL;
   GstTagList *list = gst_tag_list_new ();
+  GstBuffer *extradata = NULL;
 
   size_left = audio->size;
 
@@ -1711,167 +1750,36 @@ gst_asf_demux_add_audio_stream (GstASFDemux * asf_demux,
 
   /* Swallow up any left over data and set up the standard properties from the header info */
   if (size_left) {
-    guint8 *extradata;
-
     GST_WARNING_OBJECT (asf_demux,
         "asfdemux: Audio header contains %d bytes of codec specific data",
         size_left);
-    gst_bytestream_peek_bytes (asf_demux->bs, &extradata, size_left);
-    caps = gst_asf_demux_audio_caps (audio->codec_tag, audio, extradata,
-        &codec_name);
+    gst_bytestream_peek (asf_demux->bs, &extradata, size_left);
     gst_bytestream_flush (asf_demux->bs, size_left);
-  } else {
-    caps = gst_asf_demux_audio_caps (audio->codec_tag, audio, NULL,
-        &codec_name);
   }
+  /* asf_stream_audio is the same as gst_riff_strf_auds, but with an
+   * additional two bytes indicating extradata. */
+  caps = gst_riff_create_audio_caps_with_data (audio->codec_tag,
+      NULL, (gst_riff_strf_auds *) audio, extradata, NULL, &codec_name);
 
   /* Informing about that audio format we just added */
   gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_AUDIO_CODEC,
       codec_name, NULL);
   gst_element_found_tags (GST_ELEMENT (asf_demux), list);
-  gst_tag_list_free (list);
-  if (codec_name)
-    g_free (codec_name);
+  g_free (codec_name);
+  if (extradata)
+    gst_buffer_unref (extradata);
 
   GST_INFO ("Adding audio stream %u codec %u (0x%x)",
       asf_demux->num_video_streams, audio->codec_tag, audio->codec_tag);
 
   asf_demux->num_audio_streams++;
 
-  return gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id);
-}
+  if (!gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id))
+    return FALSE;
 
-static GstCaps *
-gst_asf_demux_video_caps (guint32 codec_fcc,
-    asf_stream_video_format * video, char **codec_name)
-{
-  GstCaps *caps = NULL;
-  gint width = 0, height = 0;
+  gst_pad_push (src_pad, GST_DATA (gst_event_new_tag (list)));
 
-  if (video != NULL) {
-    width = video->width;
-    height = video->height;
-  }
-
-  switch (codec_fcc) {
-    case GST_MAKE_FOURCC ('I', '4', '2', '0'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Raw, uncompressed I420");
-      break;
-    case GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Raw, uncompressed YUV 4:2:2");
-      break;
-
-    case GST_MAKE_FOURCC ('M', 'J', 'P', 'G'):
-      caps = gst_caps_new_simple ("video/x-jpeg", NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Motion JPEG");
-      break;
-    case GST_MAKE_FOURCC ('J', 'P', 'E', 'G'):
-      caps = gst_caps_new_simple ("video/x-jpeg", NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("JPEG Still Image");
-      break;
-    case GST_MAKE_FOURCC ('P', 'I', 'X', 'L'):
-    case GST_MAKE_FOURCC ('V', 'I', 'X', 'L'):
-      caps = gst_caps_new_simple ("video/x-jpeg", NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Miro/Pinnacle Video XL");
-      break;
-
-    case GST_MAKE_FOURCC ('D', 'V', 'S', 'D'):
-    case GST_MAKE_FOURCC ('d', 'v', 's', 'd'):
-      caps = gst_caps_new_simple ("video/x-dv",
-          "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Generic DV");
-      break;
-
-    case GST_MAKE_FOURCC ('W', 'M', 'V', '1'):
-      caps = gst_caps_new_simple ("video/x-wmv",
-          "wmvversion", G_TYPE_INT, 1, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft Windows Media 7 (WMV1)");
-      break;
-
-    case GST_MAKE_FOURCC ('W', 'M', 'V', '2'):
-      caps = gst_caps_new_simple ("video/x-wmv",
-          "wmvversion", G_TYPE_INT, 2, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft Windows Media 8 (WMV2)");
-      break;
-
-    case GST_MAKE_FOURCC ('W', 'M', 'V', '3'):
-      caps = gst_caps_new_simple ("video/x-wmv",
-          "wmvversion", G_TYPE_INT, 3, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft Windows Media 9 (WMV3)");
-      break;
-
-    case GST_MAKE_FOURCC ('M', 'P', 'G', '4'):
-      caps = gst_caps_new_simple ("video/x-msmpeg",
-          "msmpegversion", G_TYPE_INT, 41, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft MPEG-4 4.1");
-      break;
-
-    case GST_MAKE_FOURCC ('M', 'P', '4', '2'):
-      caps = gst_caps_new_simple ("video/x-msmpeg",
-          "msmpegversion", G_TYPE_INT, 42, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft MPEG-4 4.2");
-      break;
-
-    case GST_MAKE_FOURCC ('M', 'P', '4', '3'):
-      caps = gst_caps_new_simple ("video/x-msmpeg",
-          "msmpegversion", G_TYPE_INT, 43, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("Microsoft MPEG-4 4.3");
-      break;
-
-    case GST_MAKE_FOURCC ('D', 'I', 'V', '3'):
-    case GST_MAKE_FOURCC ('D', 'I', 'V', '4'):
-    case GST_MAKE_FOURCC ('D', 'I', 'V', '5'):
-      caps = gst_caps_new_simple ("video/x-divx",
-          "divxversion", G_TYPE_INT, 3, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("DivX MPEG-4 Version 3");
-      break;
-
-    case GST_MAKE_FOURCC ('D', 'I', 'V', 'X'):
-    case GST_MAKE_FOURCC ('d', 'i', 'v', 'x'):
-    case GST_MAKE_FOURCC ('D', 'X', '5', '0'):
-      caps = gst_caps_new_simple ("video/x-divx",
-          "divxversion", G_TYPE_INT, 5, NULL);
-      if (codec_name)
-        *codec_name = g_strdup ("DivX MPEG-4 Version 5");
-      break;
-
-    default:
-      GST_WARNING ("asfdemux: unknown video format " GST_FOURCC_FORMAT
-          "(0x%08x)", GST_FOURCC_ARGS (codec_fcc), codec_fcc);
-      return NULL;
-      break;
-  }
-
-  if (video != NULL) {
-    gst_caps_set_simple (caps,
-        "width", G_TYPE_INT, video->width,
-        "height", G_TYPE_INT, video->height,
-        "framerate", G_TYPE_DOUBLE, (double) 25, NULL);
-  } else {
-    gst_caps_set_simple (caps,
-        "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-        "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-        "framerate", GST_TYPE_DOUBLE_RANGE, 25.0, G_MAXDOUBLE, NULL);
-  }
-
-  return caps;
+  return TRUE;
 }
 
 static gboolean
@@ -1883,6 +1791,8 @@ gst_asf_demux_add_video_stream (GstASFDemux * asf_demux,
   gchar *name = NULL;
   char *codec_name = NULL;
   GstTagList *list = gst_tag_list_new ();
+  gint size_left = video->size - 40;
+  GstBuffer *extradata = NULL;
 
   /* Create the audio pad */
   name = g_strdup_printf ("video_%02d", asf_demux->num_video_streams);
@@ -1890,19 +1800,34 @@ gst_asf_demux_add_video_stream (GstASFDemux * asf_demux,
   g_free (name);
 
   /* Now try some gstreamer formatted MIME types (from gst_avi_demux_strf_vids) */
-  caps = gst_asf_demux_video_caps (video->tag, video, &codec_name);
+  if (size_left) {
+    GST_LOG_OBJECT (asf_demux,
+        "asfdemux: Video header contains %d bytes of codec specific data",
+        size_left);
+    gst_bytestream_peek (asf_demux->bs, &extradata, size_left);
+    gst_bytestream_flush (asf_demux->bs, size_left);
+  }
+  /* yes, asf_stream_video_format and gst_riff_strf_vids are the same */
+  caps = gst_riff_create_video_caps_with_data (video->tag, NULL,
+      (gst_riff_strf_vids *) video, extradata, NULL, &codec_name);
+  gst_caps_set_simple (caps, "framerate", G_TYPE_DOUBLE, 25.0, NULL);
   gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
       codec_name, NULL);
   gst_element_found_tags (GST_ELEMENT (asf_demux), list);
-  gst_tag_list_free (list);
-  if (codec_name)
-    g_free (codec_name);
+  g_free (codec_name);
+  if (extradata)
+    gst_buffer_unref (extradata);
   GST_INFO ("Adding video stream %u codec " GST_FOURCC_FORMAT " (0x%08x)",
       asf_demux->num_video_streams, GST_FOURCC_ARGS (video->tag), video->tag);
 
   asf_demux->num_video_streams++;
 
-  return gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id);
+  if (!gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id))
+    return FALSE;
+
+  gst_pad_push (src_pad, GST_DATA (gst_event_new_tag (list)));
+
+  return TRUE;
 }
 
 
@@ -1935,5 +1860,10 @@ gst_asf_demux_setup_pad (GstASFDemux * asf_demux,
 
   gst_element_add_pad (GST_ELEMENT (asf_demux), src_pad);
 
+  if (asf_demux->taglist) {
+    gst_pad_push (src_pad,
+        GST_DATA (gst_event_new_tag (gst_tag_list_copy (asf_demux->taglist))));
+  }
+
   return TRUE;
 }
