@@ -23,12 +23,11 @@
 
 #include <gst/gst.h>
 #include <gst/bytestream/bytestream.h>
-#include "asfheaders.h"
 
 G_BEGIN_DECLS
   
 #define GST_TYPE_ASF_DEMUX \
-  (asf_demux_get_type())
+  (gst_asf_demux_get_type())
 #define GST_ASF_DEMUX(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ASF_DEMUX,GstASFDemux))
 #define GST_ASF_DEMUX_CLASS(klass) \
@@ -57,6 +56,12 @@ typedef struct
   GstBuffer *payload;
 } asf_stream_context;
 
+typedef enum {
+  GST_ASF_DEMUX_STATE_HEADER,
+  GST_ASF_DEMUX_STATE_DATA,
+  GST_ASF_DEMUX_STATE_EOS
+} GstAsfDemuxState;
+
 struct _GstASFDemux {
   GstElement 	 element;
 
@@ -65,6 +70,12 @@ struct _GstASFDemux {
 
   GstByteStream *bs;
 
+  GstTagList *taglist;
+  GstAsfDemuxState state;
+  guint64 data_offset, num_packets, packet, data_size;
+  guint64 seek_pending;
+  gboolean seek_flush, seek_discont;
+
 #define GST_ASF_DEMUX_NUM_VIDEO_PADS 16
 #define GST_ASF_DEMUX_NUM_AUDIO_PADS 32
 #define GST_ASF_DEMUX_NUM_STREAMS 23
@@ -90,7 +101,7 @@ struct _GstASFDemux {
 
   guint32 packet_size;
   guint32 timestamp;
-  guint32 play_time;
+  guint64 play_time;
 
   guint64 preroll;
   guint64 pts;
@@ -107,11 +118,7 @@ struct _GstASFDemuxClass {
   GstElementClass parent_class;
 };
 
-GType gst_asf_demux_get_type(void);
-
-gboolean 	gst_asf_demux_plugin_init 	(GModule *module, GstPlugin *plugin);
-
-
+GType gst_asf_demux_get_type (void);
 
 G_END_DECLS
 
