@@ -242,7 +242,7 @@ gst_audio_convert_fill_one_other (gfloat ** matrix,
     GstAudioChannelPosition to_pos_r,
     GstAudioChannelPosition to_pos_c, gfloat ratio)
 {
-  gfloat in_r, out_r[2] = { 0, };
+  gfloat in_r, out_r[2];
 
   /*
    * The idea is that we add up from the input (which means that if we
@@ -350,10 +350,10 @@ gst_audio_convert_fill_others (GstAudioConvert * this)
         GST_AUDIO_CHANNEL_POSITION_REAR_CENTER, RATIO_FRONT_REAR);
   } else if (in_has_center && !out_has_center && out_has_front) {
     gst_audio_convert_fill_one_other (this->matrix,
-        &this->sinkcaps, in_c,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
-        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        &this->sinkcaps, in_r,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
         &this->srccaps, out_f,
         GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
         GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
@@ -435,25 +435,23 @@ gst_audio_convert_fill_others (GstAudioConvert * this)
 }
 
 /*
- * Normalize output values. The reason that we want to do it is related
- * to the case where we convert 5.1 to 2.0; in this case, the surround
- * channels will be mixed in the front channels, which makes their matrix
- * value > 1.0, and thus increase loudness. We don't want that. However,
- * for 2.0 to 5.1, we should not decrease volume, of course.
+ * Normalize output values.
  */
 
 static void
 gst_audio_convert_fill_normalize (GstAudioConvert * this)
 {
-  gfloat top = 1.0;
+  gfloat sum, top = 0;
   gint i, j;
 
-  /* find max. */
   for (j = 0; j < this->srccaps.channels; j++) {
+    /* calculate sum */
+    sum = 0.0;
     for (i = 0; i < this->sinkcaps.channels; i++) {
-      if (this->matrix[i][j] > top) {
-        top = this->matrix[i][j];
-      }
+      sum += fabs (this->matrix[i][j]);
+    }
+    if (sum > top) {
+      top = sum;
     }
   }
 
@@ -544,8 +542,7 @@ gst_audio_convert_mix (GstAudioConvert * this,
 {
   gint in, out, n;
   gint64 res;
-
-  gint32 *tmp = g_malloc (this->srccaps.channels * sizeof (gint32));
+  gint32 tmp[this->srccaps.channels];
   gboolean backwards = this->srccaps.channels > this->sinkcaps.channels;
 
   /* FIXME: use liboil here? */
@@ -569,6 +566,4 @@ gst_audio_convert_mix (GstAudioConvert * this,
     memcpy (&out_data[n * this->srccaps.channels], tmp,
         sizeof (gint32) * this->srccaps.channels);
   }
-
-  g_free (tmp);
 }
