@@ -124,6 +124,9 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
     case PIX_FMT_YUV420P:
       fmt = GST_MAKE_FOURCC ('I', '4', '2', '0');
       break;
+    case PIX_FMT_YVU420P:
+      fmt = GST_MAKE_FOURCC ('Y', 'V', '1', '2');
+      break;
     case PIX_FMT_YUV422:
       fmt = GST_MAKE_FOURCC ('Y', 'U', 'Y', '2');
       break;
@@ -159,6 +162,20 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       r_mask = 0x0000ff00;
       g_mask = 0x00ff0000;
       b_mask = 0xff000000;
+#endif
+      break;
+    case PIX_FMT_BGR32:
+      bpp = 32;
+      depth = 24;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0x0000ff00;
+      g_mask = 0x00ff0000;
+      b_mask = 0xff000000;
+#else
+      r_mask = 0x00ff0000;
+      g_mask = 0x0000ff00;
+      b_mask = 0x000000ff;
 #endif
       break;
     case PIX_FMT_RGBA32:
@@ -175,6 +192,22 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat pix_fmt, AVCodecContext * context)
       g_mask = 0x00ff0000;
       b_mask = 0x0000ff00;
       a_mask = 0x000000ff;
+#endif
+      break;
+    case PIX_FMT_BGRA32:
+      bpp = 32;
+      depth = 32;
+      endianness = G_BIG_ENDIAN;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+      r_mask = 0x00ff0000;
+      g_mask = 0x0000ff00;
+      b_mask = 0x000000ff;
+      a_mask = 0xff000000;
+#else
+      r_mask = 0x0000ff00;
+      g_mask = 0x00ff0000;
+      b_mask = 0xff000000;
+      a_mask = 0x000000ff;
 #endif
       break;
     case PIX_FMT_YUV410P:
@@ -431,6 +464,9 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         case GST_MAKE_FOURCC ('I', '4', '2', '0'):
           context->pix_fmt = PIX_FMT_YUV420P;
           break;
+        case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
+          context->pix_fmt = PIX_FMT_YVU420P;
+          break;
         case GST_MAKE_FOURCC ('Y', '4', '1', 'B'):
           context->pix_fmt = PIX_FMT_YUV411P;
           break;
@@ -452,7 +488,7 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
     }
   } else if (strcmp (gst_structure_get_name (structure),
           "video/x-raw-rgb") == 0) {
-    gint bpp = 0, rmask = 0, endianness = 0, amask = 0;
+    gint bpp = 0, rmask = 0, endianness = 0, amask = 0, depth = 0;
 
     if (gst_structure_get_int (structure, "bpp", &bpp) &&
         gst_structure_get_int (structure, "endianness", &endianness)) {
@@ -460,7 +496,14 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
         switch (bpp) {
           case 32:
             if (gst_structure_get_int (structure, "alpha_mask", &amask)) {
-              context->pix_fmt = PIX_FMT_RGBA32;
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+              if (rmask == 0x00ff0000)
+#else
+              if (rmask == 0x0000ff00)
+#endif
+                context->pix_fmt = PIX_FMT_BGRA32;
+              else
+                context->pix_fmt = PIX_FMT_RGBA32;
             } else {
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
               if (rmask == 0x00ff0000)
@@ -468,6 +511,13 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
               if (rmask == 0x0000ff00)
 #endif
                 context->pix_fmt = PIX_FMT_RGB32;
+
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+              if (rmask == 0x0000ff00)
+#else
+              if (rmask == 0x00ff0000)
+#endif
+                context->pix_fmt = PIX_FMT_BGR32;
             }
             break;
           case 24:
@@ -477,8 +527,13 @@ gst_ffmpeg_caps_to_pixfmt (const GstCaps * caps,
               context->pix_fmt = PIX_FMT_RGB24;
             break;
           case 16:
-            if (endianness == G_BYTE_ORDER)
+            if (endianness == G_BYTE_ORDER) {
               context->pix_fmt = PIX_FMT_RGB565;
+              if (gst_structure_get_int (structure, "depth", &depth)) {
+                if (depth == 15)
+                  context->pix_fmt = PIX_FMT_RGB555;
+              }
+            }
             break;
           case 15:
             if (endianness == G_BYTE_ORDER)
@@ -548,7 +603,7 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
   int stride, stride2;
   PixFmtInfo *pinfo;
 
-  pinfo = get_pix_fmt_info(pix_fmt);
+  pinfo = get_pix_fmt_info (pix_fmt);
 
   switch (pix_fmt) {
     case PIX_FMT_YUV420P:
@@ -573,6 +628,22 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
       picture->linesize[1] = stride2;
       picture->linesize[2] = stride2;
       return size + 2 * size2;
+      /* PIX_FMT_YVU420P = YV12: same as PIX_FMT_YUV420P, but
+       *  with U and V plane swapped. Strides as in videotestsrc */
+    case PIX_FMT_YVU420P:
+      stride = ROUND_UP_4 (width);
+      h2 = ROUND_UP_2 (height);
+      size = stride * h2;
+      stride2 = ROUND_UP_8 (stride) / 2;
+      h2 = ROUND_UP_2 (height) / 2;
+      size2 = stride2 * h2;
+      picture->data[0] = ptr;
+      picture->data[2] = picture->data[0] + size;
+      picture->data[1] = picture->data[2] + size2;
+      picture->linesize[0] = stride;
+      picture->linesize[1] = ROUND_UP_8 (stride) / 2;
+      picture->linesize[2] = ROUND_UP_8 (stride) / 2;
+      return size + 2 * size2;
     case PIX_FMT_RGB24:
     case PIX_FMT_BGR24:
       stride = ROUND_UP_4 (width * 3);
@@ -585,6 +656,8 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
     case PIX_FMT_AYUV4444:
     case PIX_FMT_RGB32:
     case PIX_FMT_RGBA32:
+    case PIX_FMT_BGR32:
+    case PIX_FMT_BGRA32:
       stride = width * 4;
       size = stride * height;
       picture->data[0] = ptr;
