@@ -3,15 +3,8 @@
 #define _RFB_RFB_H_
 
 #include <librfb/rfbdecoder.h>
+#include <librfb/rfbbytestream.h>
 #include <librfb/rfbbuffer.h>
 
-#include <gst/gst.h>
-
-G_BEGIN_DECLS
-
-GST_DEBUG_CATEGORY_EXTERN (gst_debug_rfbsrc);
-
-G_END_DECLS
-
 #endif
 
