@@ -177,7 +177,6 @@ static GstCaps *
 gst_videoscale_getcaps (GstPad * pad)
 {
   GstVideoscale *videoscale;
-  GstCaps *othercaps;
   GstCaps *caps;
   GstPad *otherpad;
   int i;
@@ -186,12 +185,11 @@ gst_videoscale_getcaps (GstPad * pad)
 
   otherpad = (pad == videoscale->srcpad) ? videoscale->sinkpad :
       videoscale->srcpad;
-  othercaps = gst_pad_get_allowed_caps (otherpad);
+  caps = gst_pad_get_allowed_caps (otherpad);
 
   GST_DEBUG_OBJECT (pad, "othercaps of otherpad %s:%s are: %" GST_PTR_FORMAT,
-      GST_DEBUG_PAD_NAME (otherpad), othercaps);
+      GST_DEBUG_PAD_NAME (otherpad), caps);
 
-  caps = gst_caps_copy (othercaps);
   for (i = 0; i < gst_caps_get_size (caps); i++) {
     GstStructure *structure = gst_caps_get_structure (caps, i);
 
@@ -265,14 +263,20 @@ gst_videoscale_link (GstPad * pad, const GstCaps * caps)
   otherstructure = gst_caps_get_structure (othercaps, 0);
   otherpar = gst_structure_get_value (otherstructure, "pixel-aspect-ratio");
   if (par && otherpar) {
-    gint num = gst_value_get_fraction_numerator (par),
-        den = gst_value_get_fraction_denominator (par),
-        onum = gst_value_get_fraction_numerator (otherpar),
-        oden = gst_value_get_fraction_denominator (otherpar);
-    gboolean keep_h,
-        w_align = (width * num * oden % (den * onum) == 0),
-        h_align = (height * den * onum % (num * oden) == 0),
-        w_inc = (num * oden > den * onum);
+    gint num, den, onum, oden;
+    gboolean keep_h, w_align, h_align, w_inc;
+
+    /* otherpar can be a list */
+    if (G_VALUE_TYPE (otherpar) == GST_TYPE_LIST)
+      otherpar = gst_value_list_get_value (otherpar, 0);
+
+    num = gst_value_get_fraction_numerator (par);
+    den = gst_value_get_fraction_denominator (par);
+    onum = gst_value_get_fraction_numerator (otherpar);
+    oden = gst_value_get_fraction_denominator (otherpar);
+    w_align = (width * num * oden % (den * onum) == 0);
+    h_align = (height * den * onum % (num * oden) == 0);
+    w_inc = (num * oden > den * onum);
 
     /* decide whether to change width or height */
     if (w_align && w_inc)
@@ -551,7 +555,7 @@ gst_videoscale_chain (GstPad * pad, GstData * _data)
   outbuf = gst_pad_alloc_buffer (videoscale->srcpad,
       GST_BUFFER_OFFSET_NONE, videoscale->to_buf_size);
 
-  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
+  gst_buffer_stamp (outbuf, buf);
 
   g_return_if_fail (videoscale->format);
   GST_LOG_OBJECT (videoscale, "format " GST_FOURCC_FORMAT,
