@@ -32,17 +32,17 @@ extern gchar *__gst_oss_plugin_dir;
 GST_DEBUG_CATEGORY (oss_debug);
 
 static gboolean
-plugin_init (GstPlugin *plugin)
+plugin_init (GstPlugin * plugin)
 {
   if (!gst_library_load ("gstaudio"))
     return FALSE;
 
-  if (!gst_element_register (plugin, "ossmixer", GST_RANK_PRIMARY,
-			     GST_TYPE_OSSELEMENT) ||
-      !gst_element_register (plugin, "osssrc", GST_RANK_PRIMARY,
-			     GST_TYPE_OSSSRC) ||
-      !gst_element_register (plugin, "osssink", GST_RANK_PRIMARY,
-			     GST_TYPE_OSSSINK)) {
+  if (!gst_element_register (plugin, "ossmixer", GST_RANK_SECONDARY,
+          GST_TYPE_OSSELEMENT) ||
+      !gst_element_register (plugin, "osssrc", GST_RANK_SECONDARY,
+          GST_TYPE_OSSSRC) ||
+      !gst_element_register (plugin, "osssink", GST_RANK_SECONDARY,
+          GST_TYPE_OSSSINK)) {
     return FALSE;
   }
 
@@ -56,14 +56,8 @@ plugin_init (GstPlugin *plugin)
   return TRUE;
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "ossaudio",
-  "OSS (Open Sound System) support for GStreamer",
-  plugin_init,
-  VERSION,
-  GST_LICENSE,
-  GST_PACKAGE,
-  GST_ORIGIN
-)
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "ossaudio",
+    "OSS (Open Sound System) support for GStreamer",
+    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
